/**Task in charge of handling user input
 * Depends on the the KEYPAD and CMD modules.
 *
 * This tasks handles user input via the KEYPAD, extracts suitable commands from entered
 * keypresses -while validating password-, and queues them for taskFsm()'s consumption.
 *
 * 20151220		0.8		Integration with the Alarm system
 * 20151220		0.2		Added CMD module usage
 * 20151218		0.1		Creation from KEYPAD example
 */

#include "FreeRTOS.h"
#include "task.h"
#include "log.h"
#include "cmd.h"
#include "fsm.h"
#include "fsmstate.h"
#include "keypad.h"
#include "alarm.h"
#include <assert.h>
#include <stdio.h>
#include <string.h>


/* callback. Policy to compute key index from a row/column pair */
uint32_t computeIndexKeypad(uint32_t a, uint32_t b, uint32_t countA, uint32_t countB)
{
	countA = countA; 				/* unused */
	return (a * countB) + b;
}

/* callback. Policy to identify a specific command */
uint32_t isExactCommand(const char* userInput, const char* cmd)
{
	return 0 == strcmp(userInput, cmd);
}

/* "Disarm" command */
Result cmdDisarm(const char* command, void* param)
{
	AlarmState* alarmState = (AlarmState*) param;
	FSM_Event event = Event_Disarm;
	xQueueSendToBack(alarmState->queueHandle, &event, portMAX_DELAY);
	return Result_OK;
}

/* "Disarm under coaction" command */
Result cmdDisarmUnderCoaction(const char* command, void* param)
{
	AlarmState* alarmState = (AlarmState*) param;
	FSM_Event event = Event_DisarmUnderCoaction;
	xQueueSendToBack(alarmState->queueHandle, &event, portMAX_DELAY);
	return Result_OK;
}

/* "Fully arm" command */
Result cmdFullyArm(const char* command, void* param)
{
	AlarmState* alarmState = (AlarmState*) param;
	FSM_Event event = Event_FullyArm;
	xQueueSendToBack(alarmState->queueHandle, &event, portMAX_DELAY);
	return Result_OK;
}

/* "Partial arm" command */
Result cmdPartialArm(const char* command, void* param)
{
	AlarmState* alarmState = (AlarmState*) param;
	FSM_Event event = Event_PartialArm;
	xQueueSendToBack(alarmState->queueHandle, &event, portMAX_DELAY);
	return Result_OK;
}

/* top-level FreeRTOS task function */
void taskCmd(void* pvParameters)
{
	assert(pvParameters != NULL);
	AlarmState* alarmState = (AlarmState*)pvParameters;

	enum { bufferSize = 10, commandDetailsSize = 4  };
	char buffers[2][bufferSize] = {{ 0 }};
	char validCommands[commandDetailsSize][10] = {{ 0 }};

	int i;
	for(i=0 ; i<commandDetailsSize ; i++)
	{
		sprintf(validCommands[i], "%s%d", alarmState->password, i+1);
	}
	CMD_details commandDetails[commandDetailsSize] =
	{
		{ isExactCommand, validCommands[0], cmdDisarm,              alarmState, "Disarm" },
		{ isExactCommand, validCommands[1], cmdFullyArm,            alarmState, "FullyArm" },
		{ isExactCommand, validCommands[2], cmdPartialArm,          alarmState, "PartialArm" },
		{ isExactCommand, validCommands[3], cmdDisarmUnderCoaction, alarmState, "DisarmUnderCoaction" }
		/* Add handling for different user codes */
	};
	CMD_cfg cmdCfg = { 0 };
	Result result = CMD_init(&cmdCfg, buffers[0], buffers[1], bufferSize, commandDetails, commandDetailsSize);
	assert(result == Result_OK);

	const KEYPAD_cfg keypadCfg =
	{
		GPIO_Port2, 0, 4,
		GPIO_Port2, 4, 3,
		3*4, "123456789*0#", ' ', computeIndexKeypad
	};
	KEYPAD_init(&keypadCfg);

	for(;;)
	{
		char keys[3*4] = { 0 };
		/*Result result = */ KEYPAD_scan(&keypadCfg, keys);
		char key = KEYPAD_getKeypress(&keypadCfg, keys);
		if(key)
		{
			CMD_processCharacter(&cmdCfg, key);
			while(key) /* eat all keypresses until no key is pressed */
			{
				/*result = */ KEYPAD_scan(&keypadCfg, keys);
				key = KEYPAD_getKeypress(&keypadCfg, keys);
			}
		}
		vTaskDelay(msToTicks(100));
	}
}
