/**Task in charge of maintaining the FSM.
 * Depends on the the FSM, FSMSTATES, and LEDCTRL modules.
 *
 * This task consumes events from taskCmd() and the SENSORS module through FSM_Alarm.queueHandle,
 * and maintains the internal state of the alarm.
 * The FSM update function is called periodically even if there are no events.
 *
 * 20151224		0.2		Splitted from main.c
 * 20151220		0.1		Message queue usage
 */

#include "FreeRTOS.h"
#include "task.h"
#include "log.h"
#include "utils.h"
#include "fsm.h"
#include "alarm.h"
#include "ledctrl.h"
#include <assert.h>


/* callback. Policy for event processing */
void processEvent(FSM_Alarm* as, FSM_Event event)
{
	FSM_SensorState* mc  = &as->sensorStateMagneticContact;
	FSM_SensorState* pir = &as->sensorStatePassiveInfrared;
	uint8_t sensorStateChanged = 1;

	switch(event)
	{
		case Event_Sensor_MagneticContact_On:
			FSM_setDetection(mc, 1);
			break;
		case Event_Sensor_MagneticContact_Off:
			FSM_setDetection(mc, 0);
			break;
		case Event_Sensor_PassiveInfrared_On:
			FSM_setDetection(pir, 1);
			break;
		case Event_Sensor_PassiveInfrared_Off:
			FSM_setDetection(pir, 0);
			break;
		default:
			sensorStateChanged = 0;
			break;
	}

	if(sensorStateChanged)
	{
		uint8_t detection = mc->detecting || pir->detecting;
		LEDCTRL_config(LEDCTRL_LedGreen, NULL, detection ? 0 : 10 , detection ? 10 : 0);
		FSM_logSensorStates(as);
	}
}

/* top-level FreeRTOS task function */
void taskFsm(void* pvParameters)
{
	assert(pvParameters != NULL);
	AlarmState* alarmState = (AlarmState*)pvParameters;

	/* alarm state initialization, step 2 */
	FSM_init(&alarmState->fsmAlarm, processEvent);
	for(;;)
	{
		FSM_Event event = Event_None;
		/* BaseType_t result =*/ xQueueReceive(alarmState->queueHandle, &event, 100);
		FSM_update(&alarmState->fsmAlarm, event);
		vTaskDelay(msToTicks(50));
	}
}
