/**FSM module. Module specific to the Alarm application.
 * Implements a Finite State Machine to represent the state of the alarm application.
 * Defines FSM events and states, sensor states, global alarm state.
 * See all the actual states implementation @ FSMSTATE module.
 *
 * Depends on FSMState and LEDCTRL.
 *
 * 20151223		0.9		Integration with alarm
 * 20151215		0.8		State refinements
 * 20151210		0.1		Creation
 */

#ifndef __FSM_H
#define __FSM_H

#include "FreeRTOS.h"
#include "queue.h"
#include <stdlib.h>
#include <stdint.h>

/* All possible FSM states */
typedef enum
{
	/*	states */
	State_Disarmed,
	State_PreArmed,
	State_Armed,
	State_PreAlarm,
	State_Alarm,
	/*	helper */
	State_MaxValue,
	State_Quit				/* testing purposes */
}	FSM_State;

/* All possible events */
typedef enum
{
	/*	unused - for testing convenience, events are mapped to 0-9 as per keypad key codes */
	Event_0_unused,
	/*	system */
	Event_Sensor_MagneticContact_On,
	Event_Sensor_MagneticContact_Off,
	Event_DisarmUnderCoaction,
	Event_Sensor_PassiveInfrared_On,
	Event_Sensor_PassiveInfrared_Off,
	Event_Disarm,
	Event_FullyArm,
	Event_PartialArm,
	/*	unused */
	Event_9_unused,
	/*	helper */
	Event_Quit, 			/* only for manual testing purposes */
	Event_None, 			/* simplifies unexistence of events */
	Event_MaxValue
}	FSM_Event;

/* sennsor state */
typedef struct
{
	char description[15];	/* this is not state, it's here for convenience */
	uint8_t detecting;		/* whether the sensor is detecting or not */
	uint8_t enabled;		/* whether the sensor is enabled (detection is meaningful) or not */
}	FSM_SensorState;

/**Alarm's FSM
 * Sensor state is here for convenience, as it will be needed in all states.
 */
typedef struct __FSM_Alarm
{
	uint32_t			timeout; 							/* in ms, timemout for time-depending states. Meaning depends on the state */
	FSM_SensorState		sensorStateMagneticContact;			/* magnetic contact sensor's state */
	FSM_SensorState		sensorStatePassiveInfrared;			/* passive infrared sensor's state */
	FSM_State			fsmState;							/* global application FSM data */
	void (*processEvent)(struct __FSM_Alarm*, FSM_Event);	/* callback to process events */
}	FSM_Alarm;

/**Alarm application state
 * This is all of the application's top-level states
 */
typedef struct
{
	FSM_Alarm			fsmAlarm;		/* FSM for the alarm system */
	QueueHandle_t		queueHandle;	/* queue for inter-task communication */
	char				password[10];	/* the user password */
}	AlarmState;

/**Signature of a state proc function
 *
 * @param	FSM_Alarm*		alarm's fsm
 * @param	FSM_Event		event to process (can be Event_None)
 */
typedef void (*FSM_AlarmStateProc)(FSM_Alarm*, FSM_Event);


/**Initializes the module
 *
 * @param	as				pointer to the alarm state
 * @param	procesEvent		callback for event processing
 */
void FSM_init(FSM_Alarm* as, void (*procesEvent)(FSM_Alarm*, FSM_Event));

/**Update function for the alarm FSM.
 * This function is called whenever a new event is received in the queue.
 * Also, it is periodically called so it can keep housekeeping of the current state.
 *
 * @param	as				alarm state
 * @param	event			a newly received event (can be Event_None)
 */
void FSM_update(FSM_Alarm* as, FSM_Event event);

/**Queries a given sensor for its detection status
 *
 * @param	sensorState		pointer to a sensor state
 * @return					nonzero if the specified sensor both is enabled and detecting
 */
uint32_t FSM_isDetecting(FSM_SensorState* sensor);

/**Set the sensor enabled value, enabling or disabling it.
 * @param	sensor			pointer to a sensor state
 * @param	value			value to set the enable member to
 */
void FSM_setEnable(FSM_SensorState* sensor, uint32_t value);

/**Set the sensor detection value
 * @param	sensor			pointer to a sensor state
 * @param	value			value to set for detection member to
 */
void FSM_setDetection(FSM_SensorState* sensor, uint32_t value);

/**One-line dump of current sensor satus (detection + enabled)
 * @param	as			alarm state
 */
void FSM_logSensorStates(FSM_Alarm* as);

/**Convenience setter to configure red/gree LEDs.
 * On/off values have the meaning as per LEDCTRL_config().
 *
 * @param	redOn		ticks to set red LED on
 * @param	redOff		ticks to set red LED off
 * @param	greenOn		ticks to set green LED on
 * @param	greenOff	ticks to set green LED off
 */
void FSM_setLeds(uint8_t redOn, uint8_t redOff, uint8_t greenOn, uint8_t greenOff);

#endif /* __FSM_H */
