/**LEDCTRL module. Module specific to the Alarm application.
 * Controls red/green (arm/detection status) LEDs as per alarm specs.
 * This module creates a FreeRTOS task upon its initialization.
 *
 * Depends on the GPIO module.
 *
 * 20151223		1.1		Patterns added
 * 20151219		1.0		Bug fixing
 * 20151216		0.9		Initial implementation
 */

#ifndef __LEDCTRL_H
#define __LEDCTRL_H

#include "gpio.h"


/**Support for two modes is implemented:
 * LEDCTRL_ModePattern
 *     When this mode is set, an ASCIIZ string is supplied, specifying '0' and '1' characters.
 *     On each tick, led state is set to on/off accordingly.
 *     After completing its cycle, mode changes to LEDCTRL_ModeIntermittency.
 *     This is meant to produce short, instantaneous user feedback when he is operating the keypad.
 * LEDCTRL_ModeIntermittency
 *     This is a cyclic state. This is now normally LEDs will be working.
 *     Led stays On for a number of ticks, then Off for a number of ticks.
 *     Remarks:
 *         Setting a LED On for 0 ticks means if will be always Off.
 *         Setting a LED Off for 0 ticks means if will be always On.
 *         Setting a LED On for 0 AND Off for 0 ticks means its current state (whatever it is) won't be changed.
 */
typedef enum
{
	/* modes */
	LEDCTRL_ModePattern,
	LEDCTRL_ModeIntermittency,
	/* helper */
	LEDCTRL_ModeMaxPatternSize = 10
}	LEDCTRL_Mode;

/* LED identifiers */
typedef enum
{
	/* ids */
	LEDCTRL_LedRed,
	LEDCTRL_LedGreen,
	/* helper */
	LEDCTRL_LedNum
}	LEDCTRL_LedId;

/**Complete configuration of a LED
 *
 * @param		ledId		either LEDCTRL_LedRed or LEDCTRL_LedGreen. Selects the LED to configure
 * @param		pattern		an ASCIIZ string containing '0' or '1' characters to set a pattern.
 *                          LEDCTRL_ModeMaxPatternSize defines the maximum acceptable pattern length.
 * @param		ticksOn		number of ticks to set the led on.
 * @param		ticksOff	number of ticks to set the led off.
 *
 * @return					see Result
 */
Result LEDCTRL_config(LEDCTRL_LedId ledId, char* pattern, uint8_t ticksOn, uint8_t ticksOff);

/**Configuration of the pattern of a LED.
 * Afer a pattern is set, mode changes to LEDCTRL_ModePattern and the pattern is executed.
 *
 * @param		ledId		either LEDCTRL_LedRed or LEDCTRL_LedGreen. Selects the LED to configure
 * @param		pattern		an ASCIIZ string containing '0' or '1' characters to set a pattern.
 *                          LEDCTRL_ModeMaxPatternSize defines the maximum acceptable pattern length.
 *
 * @return					see Result
 */
Result LEDCTRL_configPattern(LEDCTRL_LedId ledId, char* pattern);

/**Initializes the module
 * A FreeRTOS task is created when this function is called with success.
 * To remove the task from the system, use LEDCTRL_deinit().
 *
 * @param		tickDuration	milliseconds that a logical tick lasts
 * @param		redPort			GPIO port to where the red LED is connected
 * @param		redPin			GPIO pin where the red LED is connected
 * @param		greenPort		GPIO port to where the green LED is connected
 * @param		greenPin		GPIO pin to where the green LED is connected
 *
 * @return						see Result
 */
Result LEDCTRL_init(uint32_t tickDuration, GPIO_Port redPort, uint8_t redPin, GPIO_Port greenPort, uint8_t greenPin);

/**Terminates the task
 * LED states (on/off) are left as they were at the moment of the module termination.
 *
 * @return						see Result
 */
Result LEDCTRL_deinit();

#endif /* __LEDCTRL_H */
