/**SENSORS module. Module specific to the Alarm application.
 * Allows application to interact with the sensors attached.
 *
 * There are 2 sensors in the current HW configuration:
 * - Magnetic contact (MC)
 * - Passive Infrared (PIR)
 *
 * This module depends on the FSM, GPIO and GPIOINT modules.
 *
 * 20151223		1.1		Patterns added
 * 20151219		1.0		Bug fixing
 * 20151216		0.9		Initial implementation
 */

#ifndef __SENSORS_H
#define __SENSORS_H

#include "utils.h"
#include "gpio.h"
#include "gpioint.h"
#include "fsm.h"

/**Interrupt handler/callbacks (as they will be called back from the interrupt routine)
 * The sole reason for them it being public (as SENSORS_configSensor() is), is to allow module-level testing.
 *
 * They have the prototype GPIOINT_cfg.handler, as they are called by GPIOINT module. See GPIOINT for more details on this.
 *
 * Once an edge (any change) from a sensor is detected, the appropriate event is sent. Event contains information:
 * - whether it is a raising or a falling edge
 * - whether the detection change came from the MC or the PIR.
 *
 * @param	params		internally casted to AlarmState*. Allows access to the alarm's event queue.
 * @param	pinValue	will be set to either 0 or 1, according to the sensor's current detection state.
 */
void SENSORS_intrHandlerMC(void* params, uint8_t pinValue);
void SENSORS_intrHandlerPIR(void* params, uint8_t pinValue);

/**Configure one sensor
 * This function offers a much more detailed control over individual sensors, but it not mean to use in the application
 * layer. Instead, the use of SENSORT_init() is advised.
 * This function is meant for testing purposes. Do not use this unless you know what you are doing.
 *
 * @param	slot			GPIOINT slot to use. See GPIOINT for detatils.
 * @param	port			GPIO_Port to use.
 * @param	pin				GPIO pin to use to do.
 * @param	intrHandler		interrupt handler callback, as described in the GPIOINT module.
 * @param	params			user-defined data for the intrHandler. It will be up to the handler to use this parameter properly.
 * @param	sensorState		a pointer to a FSM_SensorState type. It holds the sensor state.
 * @param	enabled			whether the sensor is initially enabled or not. This will be applied to sensorState.
 * @param	description		sensor description. This will be applied to sensorState.
 *
 * @return					see Result
 */
Result SENSORS_configSensor(GPIOINT_Slot slot, GPIO_Port port , uint32_t pin, void (*intrHandler)(void*, uint8_t), void* params, FSM_SensorState* sensorState, uint8_t enabled, const char* description);

/**Initialize the module.
 * Sets the MC and PIR sensors to trigger SENSORS_intrHandlerMC() / SENSORS_intrHandlerPIR(), which are already implemented
 * in this module, and that know how to interpret the sensor outputs and queue them into the alarm system events.
 * Preconditions:
 * - Have GPIONINT module initialized.
 * - Have GPIOINT_Slot0 and GPIOINT_Slot1 free.
 *
 * @param	alarmState		global alarm state
 *
 * @return					see Result
 */
Result SENSORS_init(AlarmState* alarmState);

/**Deinitializes the module
 *
 * @return					see Result
 */
Result SENSORS_deinit(void);

#endif /* __SENSORS_H */
