#include "fsm.h"
#include "fsmstate.h"
#include "ledctrl.h"
#include "test_helpers.h"
#include <string.h>
#include "log.h"
#include "FreeRTOS.h"
#include "task.h"


uint32_t FSM_isDetecting(FSM_SensorState* sensor)
{
	return sensor->detecting && sensor->enabled;
}

void FSM_setEnable(FSM_SensorState* sensor, uint32_t value)
{
	sensor->enabled = value;
}

void FSM_setDetection(FSM_SensorState* sensor, uint32_t value)
{
	sensor->detecting = value;
}

char* FSM_getStateString(FSM_State state)
{
	static char * s [] =
	{
		"State_Disarmed",
		"State_PreArmed",
		"State_Armed",
		"State_PreAlarm",
		"State_Alarm"
	};
	return s[state];
};

void FSM_setLeds(uint8_t redOn, uint8_t redOff, uint8_t greenOn, uint8_t greenOff)
{
	/* in case LEDCTRL is not initalized, changing led configuration, should have no consequence */
	LEDCTRL_config(LEDCTRL_LedRed, NULL, redOn, redOff);
	LEDCTRL_config(LEDCTRL_LedGreen, NULL, greenOn, greenOff);
}

void FSM_UpdateLeds(FSM_Alarm* as, FSM_State current, FSM_State next)
{
	current = current; /* so far, parameter is unused */

	switch(next)
	{
		case State_Disarmed:
			if(as->sensorStateMagneticContact.detecting || as->sensorStatePassiveInfrared.detecting)
			{
				FSM_setLeds(0, 10, 0, 10);
			}
			else
			{
				FSM_setLeds(0, 10, 10, 0);
			}
			break;
		case State_PreArmed:
			FSM_setLeds(5, 5, 0, 10);
			break;
		case State_Armed:
			if(as->sensorStatePassiveInfrared.enabled)
			{
				FSM_setLeds(10, 0, 0, 10);
			}
			else
			{
				FSM_setLeds(9, 1, 0, 10);
			}
			break;
		case State_PreAlarm:
			FSM_setLeds(5, 5, 0, 10);
			break;
		case State_Alarm:
			FSM_setLeds(5, 5, 5, 5);
			break;
		default:
			break;
	}
}

void FSM_logSensorStates(FSM_Alarm* as)
{
	FSM_SensorState* mc  = &as->sensorStateMagneticContact;
	FSM_SensorState* pir = &as->sensorStatePassiveInfrared;

	LOG_log
	(
		"   sensors:   MC( d:%d e:%d )   PIR( d:%d e:%d )",
		mc->detecting, mc->enabled,
		pir->detecting, pir->enabled
	);
}

void FSM_init(FSM_Alarm* as, void (*processEvent)(FSM_Alarm*, FSM_Event))
{
	as->fsmState = State_Disarmed;
	as->timeout = 0;
	as->processEvent = processEvent;

	FSM_UpdateLeds(as, State_Disarmed, State_Disarmed);

	char tmp[30];
	LOG_log
	(
		"Initializing at %s%s%s",
		makeAnsiColors(Color_BrightWhite, Color_Black, tmp),
		FSM_getStateString(as->fsmState),
		makeAnsiResetAttributes()
	);
}

void FSM_procesState(FSM_Alarm* as, FSM_Event event)
{
	static FSM_AlarmStateProc procs [] =
	{
		&FSMSTATE_disarmed,
		&FSMSTATE_preArmed,
		&FSMSTATE_armed,
		&FSMSTATE_preAlarm,
		&FMSSTATE_alarm
	};

	FSM_State currentState = as->fsmState;
	FSM_State nextState = as->fsmState;

	switch(currentState)
	{
		case State_Disarmed:
		case State_PreArmed:
		case State_Armed:
		case State_PreAlarm:
		case State_Alarm:
			procs[currentState](as, event);
			nextState = as->fsmState;
			break;
		default:
			break;
	}

	if(currentState != nextState)
	{
		FSM_UpdateLeds(as, currentState, nextState);

		char tmp[30];
		LOG_log
		(
			"%s --> %s%s%s",
			FSM_getStateString(currentState),
			makeAnsiColors(Color_BrightWhite, Color_Black, tmp),
			FSM_getStateString(nextState),
			makeAnsiResetAttributes()
		);
	}
}

void FSM_update(FSM_Alarm* as, FSM_Event event)
{
	if(as->processEvent)
	{
		as->processEvent(as, event);
	}
	FSM_procesState(as, event);
}
