#include "fsm.h"
#include "fsmstate.h"
#include "utils.h"
#include "log.h"
#include "siscom.h"
#include "ledctrl.h"
#include "FreeRTOS.h"
#include "task.h"
#include <assert.h>


enum
{
	preArmTimeout   = 10*1000,
	PreAlarmTimeout = 10*1000
};


uint32_t getTickCount()
{
	return ticksToMs(xTaskGetTickCount());
}

uint32_t TrackTime(uint32_t* timeCount, uint32_t timeout)
{
	uint32_t tc = getTickCount();

	if(tc > 1000 + *timeCount)
	{
		*timeCount = tc;
		LOG_log("%d...", (timeout - *timeCount) / 1000);
	}

	return tc > timeout;
}

void Disarm(FSM_Alarm* as, FSM_Event event)
{
	switch(event)
	{
		case Event_DisarmUnderCoaction:
		{
			char buffer[50];
			makeAnsiString(buffer, Color_BrightRed, Color_Black, "Disarmed under coaction");
			LOG_log(buffer);
			SISCOM_notify(SiscomMessage_DisconnectedUnderCoaction);
		/*	do not break here. Carry on as if a normal disconnection */
		}
		case Event_Disarm:
			as->fsmState = State_Disarmed;
			SISCOM_notify(SiscomMessage_Disarm);
			FSM_setEnable(&as->sensorStateMagneticContact, 0);
			FSM_setEnable(&as->sensorStatePassiveInfrared, 0);
			break;
		default:
			assert(0);
			break;
	}
}

void Alarm(FSM_Alarm* as, SISCOM_Message siscomMessage)
{
	char buffer[50];
	makeAnsiString(buffer, Color_BrightRed, Color_Black, "*** Alarm went off ***");
	LOG_log(buffer);
	as->fsmState = State_Alarm;

	SISCOM_notify(siscomMessage);
}

void FSMSTATE_disarmed(FSM_Alarm* as, FSM_Event event)
{
	uint8_t canArm = (as->sensorStateMagneticContact.detecting == 0) && (as->sensorStatePassiveInfrared.detecting == 0);
	uint8_t warnCannotArm = !canArm;
	switch(event)
	{
		case Event_FullyArm:
			if(canArm)
			{
				FSM_setEnable(&as->sensorStateMagneticContact, 1);
				FSM_setEnable(&as->sensorStatePassiveInfrared, 1);
				as->fsmState = State_PreArmed;
				as->timeout = getTickCount() + preArmTimeout;
				LOG_log("System will be fully armed in %ds", preArmTimeout/1000);
			}
			break;
		case Event_PartialArm:
			if(canArm)
			{
				FSM_setEnable(&as->sensorStateMagneticContact, 1);
				FSM_setEnable(&as->sensorStatePassiveInfrared, 0);
				as->fsmState = State_PreArmed;
				as->timeout = getTickCount() + preArmTimeout;
				LOG_log("System will be partially armed in %ds", preArmTimeout/1000);
			}
			break;
		default:
			warnCannotArm = 0;
			break;
	}

	if(warnCannotArm)
	{
		char buffer[100];
		makeAnsiString(buffer, Color_BrightRed, Color_Black, "Cannot Arm. Installation is not secure.");
		LOG_log("%s", buffer);
	//	FSM_setLeds(0, 10, 5,  5);
		LEDCTRL_configPattern(LEDCTRL_LedGreen, "101010101");
	/*	LEDCTRL_configPattern
		(
			LEDCTRL_LedGreen,
			warnCannotArm ? "101010101" : NULL,
			canArm ? 10 :  0,
			canArm ?  0 : 10
		);	*/
	}
}

void FSMSTATE_preArmed(FSM_Alarm* as, FSM_Event event)
{
	static uint32_t timeCount = 0;

	switch(event)
	{
		case Event_DisarmUnderCoaction:
		case Event_Disarm:
			Disarm(as, event);
			timeCount = 0;
			break;
		default:
			if(TrackTime(&timeCount, as->timeout))
			{
				char buffer[50];
				makeAnsiString(buffer, Color_BrightRed, Color_Black, "Alarm is armed.");
				LOG_log("%s %s", buffer, "Sensor states are:");
				FSM_logSensorStates(as);
				as->fsmState = State_Armed;
				SISCOM_notify(SiscomMessage_Arm);
				timeCount = 0;
			}
			break;
	}
}

void FSMSTATE_armed(FSM_Alarm* as, FSM_Event event)
{
	switch(event)
	{
		case Event_DisarmUnderCoaction:
		case Event_Disarm:
			Disarm(as, event);
			break;
		default:
			if(FSM_isDetecting(&as->sensorStatePassiveInfrared))
			{
				Alarm(as, SiscomMessage_AlarmPIR);
			}
			else if(FSM_isDetecting(&as->sensorStateMagneticContact))
			{
				as->timeout = getTickCount() + PreAlarmTimeout;
				as->fsmState = State_PreAlarm;
			}
			break;
	}
}

void FSMSTATE_preAlarm(FSM_Alarm* as, FSM_Event event)
{
	static uint32_t timeCount = 0;

	switch(event)
	{
		case Event_DisarmUnderCoaction:
		case Event_Disarm:
			Disarm(as, event);
			timeCount = 0;
			break;
		default:
			if(TrackTime(&timeCount, as->timeout))
			{
				Alarm(as, SiscomMessage_AlarmMC);
				timeCount = 0;
			}
			break;
	}
}

void FMSSTATE_alarm(FSM_Alarm* as, FSM_Event event)
{
	switch(event)
	{
		case Event_DisarmUnderCoaction:
		case Event_Disarm:
			Disarm(as, event);
			break;
		default:
			break;
	}
}
