#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "ledctrl.h"
#include "gpio.h"
#include "utils.h"
#include "FreeRTOS.h"
#include "task.h"
#include <string.h>
#include <assert.h>


typedef struct
{
/*  config & state */
	GPIO_Port		port;
	uint8_t			pin;
	LEDCTRL_Mode	mode;
	uint32_t		tickCount;
/*  params for pattern mode */
	char			pattern[LEDCTRL_ModeMaxPatternSize];
/*  params for intermittency mode */
	uint8_t			ticksOn;
	uint8_t			ticksOff;
}	LEDCTRL_ledState;


xSemaphoreHandle LEDCTRL_semaphore                  = NULL;
xTaskHandle LEDCTRL_taskHandle                      = NULL;
uint32_t LEDCTRL_tickDuration                       = 100;		/* tick duration in ms */
LEDCTRL_ledState LEDCTRL_ledStates[LEDCTRL_LedNum]  = {{ 0 }};  /* red, green led states */


uint8_t LEDCTRL_isInitialized()
{
	return LEDCTRL_taskHandle != NULL;
}

Result LEDCTRL_config(LEDCTRL_LedId ledId, char* pattern, uint8_t ticksOn, uint8_t ticksOff)
{
	if(!LEDCTRL_isInitialized())
	{
		return Result_Failed_NotInitialized;
	}
	if(ledId > LEDCTRL_LedNum)
	{
		return Result_Failed_InvalidParams;
	}
	if(pattern && (strlen(pattern) >= LEDCTRL_ModeMaxPatternSize))
	{
		return Result_Failed_InvalidParams;
	}

	xSemaphoreTake(LEDCTRL_semaphore, portMAX_DELAY);

	LEDCTRL_ledState* led = &LEDCTRL_ledStates[ledId];
	if(pattern)
	{
		led->mode = LEDCTRL_ModePattern;
		strcpy(led->pattern, pattern);
	}
	else
	{
		led->mode = LEDCTRL_ModeIntermittency;
		led->pattern[0] = 0;
	}
	led->ticksOn = ticksOn;
	led->ticksOff = ticksOff;
	led->tickCount = 0;

	xSemaphoreGive(LEDCTRL_semaphore);
	return Result_OK;
}

Result LEDCTRL_configPattern(LEDCTRL_LedId ledId, char* pattern)
{
	if(!LEDCTRL_isInitialized())
	{
		return Result_Failed_NotInitialized;
	}
	LEDCTRL_ledState* led = &LEDCTRL_ledStates[ledId];
	uint8_t ticksOn = led->ticksOn;
	uint8_t ticksOff = led->ticksOff;
	return LEDCTRL_config(ledId, pattern, ticksOn, ticksOff);
}

void LEDCTRL_updateLedPattern(LEDCTRL_ledState* led)
{
	if((led->tickCount < LEDCTRL_ModeMaxPatternSize) && (led->pattern[led->tickCount] != 0))
	{
		uint8_t value = led->pattern[led->tickCount] != '0';
		GPIO_setPinValue(led->port, led->pin, value);

		led->tickCount++;
		if((led->tickCount >= LEDCTRL_ModeMaxPatternSize) || (led->pattern[led->tickCount] == 0))
		{
			led->mode = LEDCTRL_ModeIntermittency;
			led->tickCount = 0;
		}
	}
	else
	{
		led->mode = LEDCTRL_ModeIntermittency;
		led->tickCount = 0;
	}
}

void LEDCTRL_updateLedIntermitency(LEDCTRL_ledState* led)
{
	uint32_t c = led->tickCount % (uint32_t)(led->ticksOn + led->ticksOff);

	if(led->ticksOn > 0)
	{
		if(c == 0)
		{
			GPIO_setPinValue(led->port, led->pin, 1);
		}
		else if(c == led->ticksOn && led->ticksOff > 0)
		{
			GPIO_setPinValue(led->port, led->pin, 0);
		}
	}
	else
	{
		if(c == 0 && led->ticksOff > 0)
		{
			GPIO_setPinValue(led->port, led->pin, 0);
		}
	/* note that if led->ticksOn == 0 and led->ticksOff == 0, les state is never updated */
	}
	led->tickCount++;
}

Result LEDCTRL_updateLed(LEDCTRL_ledState* led)
{
	if(!LEDCTRL_isInitialized())
	{
		return Result_Failed_NotInitialized;
	}
	switch(led->mode)
	{
		case LEDCTRL_ModePattern:
			LEDCTRL_updateLedPattern(led);
			break;
		case LEDCTRL_ModeIntermittency:
			LEDCTRL_updateLedIntermitency(led);
			break;
		default:
			assert(0);
	}
	return Result_OK;
}

void LEDCTRL_proc(void* params)
{
	while(1)
	{
		xSemaphoreTake(LEDCTRL_semaphore, portMAX_DELAY);
		int i;
		for(i=0 ; i<LEDCTRL_LedNum ; i++)
		{
			LEDCTRL_updateLed(&LEDCTRL_ledStates[i]);
		}
		xSemaphoreGive(LEDCTRL_semaphore);
		vTaskDelay(msToTicks(LEDCTRL_tickDuration));
	}
}

Result LEDCTRL_init(uint32_t tickDuration, GPIO_Port redPort, uint8_t redPin, GPIO_Port greenPort, uint8_t greenPin)
{
	if(LEDCTRL_isInitialized())
	{
		return Result_Failed_AlreadyInitialized;
	}
	if(redPort == greenPort && redPin == greenPin)
	{
		return Result_Failed_InvalidParams;
	}
	if(Result_OK != GPIO_configPinAsOutput(redPort, redPin))
	{
		return Result_Failed_InvalidParams;
	}
	if(Result_OK != GPIO_configPinAsOutput(greenPort, greenPin))
	{
		return Result_Failed_InvalidParams;
	}

	LEDCTRL_semaphore = xSemaphoreCreateMutex();
	if(LEDCTRL_semaphore == NULL)
	{
		return Result_Failed; /* semaphore initialization failed */
	}

	LEDCTRL_tickDuration = tickDuration;

	LEDCTRL_ledStates[LEDCTRL_LedRed].port = redPort;
	LEDCTRL_ledStates[LEDCTRL_LedRed].pin = redPin;
	LEDCTRL_ledStates[LEDCTRL_LedGreen].port = greenPort;
	LEDCTRL_ledStates[LEDCTRL_LedGreen].pin = greenPin;

	LEDCTRL_config(LEDCTRL_LedRed, NULL, 0, 0);
	LEDCTRL_config(LEDCTRL_LedGreen, NULL, 0, 0);

	BaseType_t createResult = xTaskCreate(LEDCTRL_proc, "LedCtrlProc", 100, NULL, tskIDLE_PRIORITY+3, &LEDCTRL_taskHandle);
	if(createResult != pdTRUE)
	{
		vSemaphoreDelete(LEDCTRL_semaphore);
		return Result_Failed_NotEnoughMemory;
	}
	return Result_OK;
}

Result LEDCTRL_deinit()
{
	if(LEDCTRL_taskHandle == NULL)
	{
		return Result_Failed_NotInitialized;
	}

	vTaskDelete(LEDCTRL_taskHandle);
	eTaskState state1 = eTaskGetState(LEDCTRL_taskHandle);
	vTaskDelay(msToTicks(LEDCTRL_tickDuration*2));
	eTaskState state2 = eTaskGetState(LEDCTRL_taskHandle);

	LEDCTRL_taskHandle = NULL;
	vSemaphoreDelete(LEDCTRL_semaphore);
	return (eDeleted == state1 && eReady == state2) ? Result_OK : Result_Failed;
}
