/**CMD module
 * Handles command recognition from a sequence of entered characters.
 * Also, handles user-defined command procedure selection and invocation based on the command.
 * Command procedures have a context parameter, allowing easy integration with other subsystems.
 *
 * This is not a singleton module. Many instances of CMD_cfg are allowed.
 * Depends on CIRCULARBUFFER module.
 *
 * 20151213		0.9		Implementation
 */

#ifndef __CMD_H
#define __CMD_H

#include "circularbuffer.h"
#include "utils.h"


/* details for a command */
typedef struct
{
	uint32_t (*isCommand)(const char*, const char*);	/* callback to recognise a command */
	const char* command;								/* command */
	Result (*proc)(const char*, void*);					/* command procedure, will be invoked if a command is acknowledged */
	void* params;										/* command procedure parameters, allows contextualization of parameters */
	char* description;									/* command description */
}	CMD_details;


/* module configuration */
typedef struct
{
	char*			cmdBuffer;							/* auxiliary buffer, user-supplied */
	char*			workBuffer;							/* another auxiliary buffer, user-supplied */
	uint32_t		bufferSize;							/* size for both cmdBuffer and workBuffer */
	CIRCULARBUFFER_data	cb;								/* circular buffer */
	CMD_details*	cmdDetails;							/* array of possible commands */
	uint32_t		cmdDetailsSize;						/* size of cmdDetails */
}	CMD_cfg;


/**Initialize a CMD instance.
 *
 * @param	cfg				the CMD_cfg that holds the command state
 * @param	buffer1			one work buffer. Size must be bufferSize. It must be allocated by the user
 * @param	buffer2			another work buffer. Size must be bufferSize. It must be allocated by the user
 * 							both buffer1 and buffer2 are needed to do internal processing
 * @param	bufferSize		size of both buffer1 and buffer2
 * @param	cmdDetails		array of cmdDetails structure
 * @param	cmdDetailsSize	number of items in cmdDetails
 *
 * @return					see Result
 */
Result CMD_init(CMD_cfg* cfg, char* buffer1, char* buffer2, uint32_t bufferSize, CMD_details* cmdDetails, uint32_t cmdDetailsSize);

/**Process a character.
 * This involves adding the character to the internal circular buffer, checking whether there is a valid command in the
 * buffer, and if so, run the associated command
 *
 * @param	cfg		the CMC_cfg that holds the command state
 * @param	c		the newly entered character
 *
 * @return			see Result
 */
Result CMD_processCharacter(CMD_cfg* cfg, char c);

#endif /* __CMD_H */
