/**GPIOINT module
 * Extensio to GPIO module to support for external GPIO interrupts
 * Only Port 0 and 2 can be used as sensitive interrupt sources.
 * Only rising and/or falling edge interrupts can be configured.
 *
 *	20151128	0.92	 Support for edged interrupts
 *	20151126	0.90	 Initial version
 */

#ifndef __GPIOINT_H
#define __GPIOINT_H

#include "utils.h"
#include "gpio.h"

/**Available slots
 * Instead of receving handles, it's responsability to the user to reserve and use the desired slot.
 * This has been convenient so far. ALthough a handle system might be implemented in the future.
 */
typedef enum
{
	GPIOINT_Slot0,
	GPIOINT_Slot1,
	GPIOINT_Slot2,
	GPIOINT_Slot3,
	GPIOINT_Slot4,
	GPIOINT_Slot5,
	GPIOINT_Slot6,
	GPIOINT_Slot7,
	GPIOINT_SlotNum = 8
}	GPIOINT_Slot;

/* Edge configuration that will trigger the interrupt */
typedef enum
{
	GPIOINT_EdgeKind_None,
	GPIOINT_EdgeKind_Raising,
	GPIOINT_EdgeKind_Falling,
	GPIOINT_EdgeKind_RaisingORFalling
}	GPIOINT_EdgeKind;

/* Configuration for a single external GPIO pin interruption */
typedef struct
{
	void (*handler)(void*, uint8_t);		/* interrupt handler */
	void*				params;				/* user-defined interrupt handler parameters */
	GPIO_Port			port;				/* port involved. Only 0 or 2 are valid values */
	uint32_t			pin;				/* pin involved */
	GPIOINT_EdgeKind	edgeKind;			/* kind of edge to that must trigger the interrupt */
}	GPIOINT_cfg;

/* Checks whether the module is initialized */
uint8_t GPIOINT_isInitialized(void);

/* Initialize the module */
Result GPIOINT_init(void);

/* Deinitializes the module */
Result GPIOINT_deinit(void);

/**Set a external GPIO pin interruption
 *
 * @param		slot		a free user defined slot.
 * @param		userCfg		configuration parameters for the new slot
 *
 * @return					see Result
 */
Result GPIOINT_set(GPIOINT_Slot slot, GPIOINT_cfg* userCfg);

/**Unset (clear) a external GPIO pin interruption
 *
 * @param		slot		a previously set user defined slot.
 *
 * @return					see Result
 */
Result GPIOINT_unset(GPIOINT_Slot slot);

#endif /* __GPIOINT_H */
