/**KEYPAD module
 * Support for generic matrix KEYBOARD
 *
 * Depends on GPIO module.
 * A pending improvement would be remove the need to actively scan while there are no keypresses.
 * Instead, an initial interrupt could trigger a scan operation, which would last for some time.
 *
 *	20151212	0.9     Fixes, cleanup
 *	20151122	0.6     Prelimnary version, first Release
 */

#ifndef __KEYPAD_H
#define __KEYPAD_H

#include "utils.h"
#include "gpio.h"

/**Configuration for a KEYPAD
 * You can think of (A,B axis) = (rows, columns) or the other way around
 */
typedef struct
{
	GPIO_Port		portA;			/* port where A axis is connected */
	uint32_t		startA;			/* starting pin for A axis */
	uint32_t		countA;			/* number of pins in A axis */

	GPIO_Port		portB;			/* port where B axis is connected */
	uint32_t		startB;			/* starting pin for B axis */
	uint32_t		countB;			/* number of pins in B axis */

	uint32_t		keyListSize;	/* size of keyList array */
	char*			keyList;		/* array containing a character for each key in keypad */
	char			noKey;			/* default 'no character' */

									/* Translation callback from a keypress @(A, B) axis to a index in keyList */
	uint32_t		(*computeIndexCallback)(uint32_t, uint32_t, uint32_t, uint32_t);
}	KEYPAD_cfg;

/**Module initialization
 *
 * @param	cfg		keypad configuration
 *
 * @return			see Result
 */
Result KEYPAD_init(const KEYPAD_cfg* cfg);

/**Scan for pressed keys
 * It returns an array of keys currently pressed.
 * Array must have the length as KEYPAD_cfg.keyListSize, and for each character in KEYPAD_cfg.keyList,
 * the equivalent position in the keys arrays will be initialized with that character if that key is pressed,
 * or set to the KEYPAD_cfg.noKey character.
 *
 * This function allows to read a number of simultaneously pressed characters in one scan. But depending on
 * the keypad HW configuration that may no the possible.
 *
 * @param	cfg		keypad configuration
 * @param	keys	user-allocated, array where to return set keys pressed
 *
 * @return			see Result
 */
Result KEYPAD_scan(const KEYPAD_cfg* cfg, char* keys);

/**Get one-key-only, clean, keypress from the key buffer read from KEYPAD_scan()
 *
 * @param	cfg		keypad configuration
 * @param	keys	array of keys, as returned by KEYPAD_scan()
 * 					it should be 0 terminated
 * @return			0 if no single keypress can be identified, or
 *					1 if exactly 1 keypress was issued.
 */
char KEYPAD_getKeypress(const KEYPAD_cfg* cfg, const char* keys);

#endif /* __KEYPAD_H */
