/**
 * PRINTF printf()-like interface for formatted output to UART
 * Depends on UART module
 *
 * 20151103		0.9		Cleanup. Documentation updated
 * 20151012		0.8		PRINTF_semaphore added
 * 20151011		0.7 	Creation
 */

#ifndef __PRINTF_H_
#define __PRINTF_H_

#include "FreeRTOS.h"
#include "semphr.h"
#include "uart.h"
#include "utils.h"
#include <stdarg.h>


/**Initializes the module
 *
 * @param	portNum		port that we want to initialize
 *
 * @return	see Result
 */
Result PRINTF_init(UART_port port);

/**Deinitializes the module
 *
 * @param	portNum		port that we want to deinitialize
 *
 * @return	see Result
 */
Result PRINTF_deinit(UART_port port);

/**Checks if the module is initialized
 *
 * @param	portNum	port that we want to initialize
 *
 * @return	0/1 	on non-initialization/initialization
 */
int PRINTF_isInitialized(UART_port port);

/**Outs a string to an UART port
 *
 * @param	portNum		UART port to use
 * @param	format		same as printf()
 * @param	...			same as printf()
 *
 * @return	see Result
 */
Result PRINTF_print(UART_port port, const char* format, ...);

/**Outs a string to an UART port
 *
 * @param	portNum		UART port to use
 * @param	format		same as vprintf()
 * @param	argList		same as vprintf()
 *
 * @return	see Result
 */
Result PRINTF_printVa(UART_port port, const char* format, va_list argList);

/**Active waits while polling UART for a string terminated in CR
  *
 * @param	scanPort	UART port from where to read data
 * @param	echoPort	Alternative UART port where to echo data
 * 						Specify UART_PortNone to disable echo
 * @param	result		buffer where result will be saved, if any
 * @param	maxSize		Maximum number of characters to read
 * 						Once such number is reached, function will return
 *
 * @return	see Result
 */
Result PRINTF_scanFromHuman(UART_port scanPort, UART_port echoPort, char* result, size_t maxSize);

/**Active waits while polling UART for a string terminated in CR
 *
 * @param	port		UART port from where to read data
 * @param	result		buffer where result will be saved, if any
 * @param	maxSize		Maximum number of characters to read
 * 						Once such number is reached, function will return
 * @param	timeout		Maximum amount of time to wait
 *
 * @return	see Result
 */
Result PRINTF_scanFromDevice(UART_port port, char* result, size_t maxSize, uint32_t timeout);


#endif /* __PRINTF_H_ */
