/**UART module
 * Support for UART usage
 * Two operations modes are supported. With interrupts and without interrupts (polling on reads)
 * So far, there is no support for UART2, leaving only UARTS 0,1,3 usable
 *
 * This is refactored rip-off of "Simple UART echo for LPCXpresso 1700"'s uarttest.c
 *
 *	20151011	1.10	 Dependency of taskHandleReadCmd
 *	20100701	1.01    Added support for UART3, tested on LPCXpresso 1700
 *	20090527	1.00    Prelimnary version, first Release
 */

#ifndef __UART_H
#define __UART_H

#include "LPC17xx.h"
#include "utils.h"


enum { UART_bufferSize = 0x40 };

typedef enum
{
	UART_port0		= 0,	/* UART 0 */
	UART_port1		= 1,	/* UART 1 */
	UART_port2		= 2,	/* UART 2 - Unsupported. Do not use */
	UART_port3		= 3,	/* UART 3 */
	UART_portNum	= 4,	/* number of UART ports */
	UART_portNone	=-1		/* special value to signify that no usart is used */
}	UART_port;

typedef enum				/* operation mode. this is a per-UART setting */
{
	UART_modeNotInitialized,
	UART_modeWithoutInterrupts,
	UART_modeWithInterrupts
}	UART_mode;

typedef struct				/* per-UART state */
{
	UART_mode volatile mode;
	LPC_UART_TypeDef* volatile lpc;
/*	below values are used in interrupt mode only */
	uint8_t volatile txEmpty;
	uint32_t volatile status;
	uint8_t volatile buffer[UART_bufferSize];		/* TODO: change to a char*, add a bufferSize member to allow differently bufferered sizes */
	uint32_t volatile count;
}	UART_state;


/** Initialize the module
 *
 * @param portNum		the port to initialize
 * @param Baudrate		the baud rate for communication
 *
 * @return				see Result
 */
Result UART_init(UART_port port, uint32_t Baudrate, UART_mode mode);

/** Deinitialize the module
 *
 * @param portNum		the port to initialize
 *
 * @return				see Result
 */
Result UART_deinit(UART_port port);

/** Check if module is initialized for the given port
 *
 * @param portNum		the port to check initialization for
 *
 * @return				see Result
 */
int UART_isInitialized(UART_port port);


/** Send a string through the specified port
 *
 * @param portNum		the port to use
 * @param BufferPtr		pointer to data buffer
 * @param Length		size of data buffer
 *
 */
void UART_send(UART_port port, char* BufferPtr, uint32_t Length );

/** Read a character from the specified port
 *	This blocks indefinitely until a character is read
 *
 * @param portNum		the port to use
 *
 * @return				character read
 */
char UART_getCharacter(UART_port port);

/** Read a character from the specified port, with a timeout
 *
 * @param portNum		the port to use
 *
 * @return				character read. 0 if no character was read and timeout expired
 */
char UART_getCharacterWithTimeout(UART_port port, uint32_t timeoutMs);

/** Eats character from the specified port, until timeout expires
 *
 * @param portNum		the port to use
 *
 * @return				see Result
 */
Result UART_clear(UART_port port, uint32_t timeout);


#endif /* end __UART_H */
