/**
 * WIFLY module
 * Interface with a "Wifly radio module"
 * Encapsulates many common functions
 * Depends on PARSING, UART, PRINTF, LOG modules
 *
 * 20151108		0.9		Many commands added
 * 20151029		0.8		Get IP, generic command added
 * 20151027		0.7		Init, deinit, reboot added
 * 20151022		0.01	Creation
 */

#ifndef __WIFLY_H 
#define __WIFLY_H

#include "LPC17xx.h"
#include "utils.h"


/**Initializes the module
 *
 * @param	port	UART port to use
 * @param	reboot	0/1. 1: issue a reboot command sequence
 *
 * @return	see Result
 */
Result WIFLY_init(uint32_t port, uint32_t reboot);

/**Deinitializes the module
 *
 * @return	see Result
 */
Result WIFLY_deinit();

/**Generic command executor
 * Starts the command mode, sends the specified command, collects the response, exits command mode
 *
 * @param	cmd			Wifly command to execute
 * 						Must end with a \r - many of commands could be sent, if separated by \r
 * @param	buffer		Result of the specified command
 * 						This is only valid if return code is Result_OK
 * @param	bufferSize	Size of the buffer pointed by result
 * @param	cmdTimeout	Maximum amount of time to wait for an answer to the command
 *
 * @return	see Result
 */
Result WIFLY_genericCommand(const char* cmd, char* buffer, size_t bufferSize, uint32_t timeout);

/**Join command
 *
 * @param	ssid		SSID to join
 * @param	pwd			Password for the ssid
 *
 * @return	see Result
 */
Result WIFLY_join(const char* ssid, const char* pwd);

/**Reboots the module
 *
 * @return	see Result
 */
Result WIFLY_reboot();

/**Gets the ip where we are connected
 *
 * @param	ip		Buffer to receive the ip
 * 					It is up to the user that ip points to a buffer large enough to accomodate an IPv4 (16 characters)
 *
 * @return	see Result
 */
Result WIFLY_getIp(char* ip);

/**Gets the WiFi channel where we are connected to
 *
 * @param	channel		Buffer to receive the channel
 * 						It is up to the user that channed points to a buffer large enough to accomodate the channel number (2 characters)
 *
 * @return	see Result
 */
Result WIFLY_getChannel(char* channel);

/**Gets the last received signal strength
 *
 * @param	rssi		Buffer to receive the RSSI
 * 						It is up to the user that channed points to a buffer large enough to accomodate the RSSI value
 *
 * @return	see Result
 */
Result WIFLY_getRSSI(char* rssi);

/**Sends an HTTP GET request
 *
 * @param	server			URL host name
 * @param	remainderURL	URL path name + parameters
 * @param	response		Buffer to receive response
 * @param	responseSize	Size of Buffer
 *
 * @return	see Result
 */
Result WIFLY_sendHttpGetRequest(const char *server, const char* remainderUrl, char* response, size_t responseSize);

/**Gets the transmit power
 *
 * @param	txPower			receives the current power level (in dBm)
 *
 * @return	see Result
 */
Result WIFLY_getTxPower(int* txPower);

/**Sets the transmit power
 *
 * @param	txPower			desired power level (in dBm)
 *
 * @return	see Result
 */
Result WIFLY_setTxPower(int txPower);

/**Gets the uptime
 *
 * @param	txPower			receives the uptime (in s)
 *
 * @return	see Result
 */
Result WIFLY_getUptime(int* uptime);

/**Pings a remote host
 *
 * @param	address			Host's IP to ping
 * @param	value			Number of pings to send
 * 							This parameter can be abused. Check the manual for extended ping functionality
 * @return	see Result
 */
Result WIFLY_ping(const char* address, const char* value);


#endif /* end __WIFLY_H */
