#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "queue.h"
#include "printf.h"
#include "uart.h"
#include "utils.h"
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>


/* provides synchronization for concurrent access */
xSemaphoreHandle PRINTF_semaphores[UART_portNum] = { 0 };


int PRINTF_isInitialized(UART_port port)
{
	return PRINTF_semaphores[port] != NULL;
}

Result PRINTF_init(UART_port port)			/*  TODO: initialization order vs UART? - see PRINTF_semaphore usage */
{
	if(PRINTF_isInitialized(port))
	{
		return Result_Failed_AlreadyInitialized;
	}
	if(UART_isInitialized(port))
	{
		return Result_Failed; /* UART can't be initialized, as its initialization depends on PRINTF's */
	}
	PRINTF_semaphores[port] = xSemaphoreCreateMutex();
/*	PRINTF_semaphores[port] = xSemaphoreCreateCounting(20, 0); */
/*	PRINTF_semaphores[port] = xSemaphoreCreateBinary(); */
	if(PRINTF_semaphores[port] == NULL)
	{
		return Result_Failed; /* semaphore initialization failed */
	}
	return Result_OK;
}

Result PRINTF_deinit(UART_port port)
{
	if(!PRINTF_isInitialized(port))
	{
		return Result_Failed_NotInitialized;
	}

	xSemaphoreTake(PRINTF_semaphores[port], portMAX_DELAY);
	xSemaphoreHandle aux = PRINTF_semaphores[port];
	PRINTF_semaphores[port] = 0;
	xSemaphoreGive(aux);
	vSemaphoreDelete(aux);
	return Result_OK;
}

Result PRINTF_print(UART_port port, const char* format, ...)
{
	if(!PRINTF_isInitialized(port))
	{
		return Result_Failed_NotInitialized;
	}

    va_list argList;
	va_start(argList, format);
	Result result = PRINTF_printVa(port, format, argList);
	va_end(argList);
	return result;
}

Result PRINTF_printVa(UART_port port, const char* format, va_list argList)
{
	if(!PRINTF_isInitialized(port))
	{
		return Result_Failed_NotInitialized;
	}

	Result result = Result_Failed_NotInitialized;
	xSemaphoreTake(PRINTF_semaphores[port], portMAX_DELAY);
	enum { formattedMaxSize = 256 };
	char formatted[formattedMaxSize] = {0};
	int requiredSize = vsnprintf(formatted, formattedMaxSize, format, argList);
	if(requiredSize >=0 && requiredSize < formattedMaxSize)
	{
		UART_send(port, formatted, strlen(formatted));
		result = Result_OK;		/* TODO: weak; we are not sure that UART_send() succeeded */
	}
	else
	{
		result = Result_Failed; /* not enough memory? */
	}
	xSemaphoreGive(PRINTF_semaphores[port]);
    return result;
}

/* Active waits while polling for a string token
 * String is terminated on a CR.
 * ESC cancels input
 *
 * Can echo back the entered character
 *
 * @param scanPortNum		port to scan for characters
 * @param echoPortNum		port to echo characters read. If PRINTF_noEcho is specified, echo is disabled.
 * @param result			returned string, if any
 * @param maxSize			max allowed size to write result
 */
Result PRINTF_scanFromHuman(UART_port  scanPort, UART_port echoPort, char* result, size_t maxSize)
{
	if(!PRINTF_isInitialized(scanPort))
	{
		return Result_Failed_NotInitialized;
	}
	if((echoPort != UART_portNone) && (!PRINTF_isInitialized(echoPort)))
	{
		return Result_Failed_NotInitialized;
	}

	size_t i=0;
	while(i < maxSize)
	{
		char c = UART_getCharacter(scanPort);
		if(c && (echoPort != UART_portNone))
		{
			PRINTF_print(echoPort, "%c", c);		/* TODO: CR translated to CRLF? */
		}
		switch(c)
		{
			case '\r':				/*	CR: accept input */
				result[i] = 0;
				return Result_OK;
				break;
			case '\n':				/*	LF: (in case input is redirected from a file with LF) eat the character */
				break;
			case 0:					/*	\0 (no character available): release CPU */
				vTaskDelay(msToTicks(100));
				break;
			case 0x1B:				/*	ESC: cancel input */
				result[0] = 0;
				return Result_Failed_Cancelled;
				break;
			default:				/*	other: add character to buffer */
				result[i++] = c;
				break;
		}
	}
	return Result_Failed_NotEnoughMemory;
}

Result PRINTF_scanFromDevice(UART_port port, char* result, size_t maxSize, uint32_t timeoutMs) /*timeout in ms*/
{
	if(!PRINTF_isInitialized(port))
	{
		return Result_Failed_NotInitialized;
	}

	uint32_t timeoutTicks = xTaskGetTickCount() + msToTicks(timeoutMs);
	size_t i=0;
	while(i < maxSize)
	{
		char c = UART_getCharacterWithTimeout(port, 20);
		switch(c)
		{
			case '\0':					/*	\0 (no character available): release CPU */
				if(xTaskGetTickCount() > timeoutTicks)
				{
					result[i] = 0;
					return Result_OK; /* Result_Failed_Timeout; */
				}
				break;
			default:
				result[i++] = c;
				timeoutTicks = xTaskGetTickCount() + msToTicks(timeoutMs);
				break;
		}
	}
	return Result_Failed_NotEnoughMemory;
}
