/***********************************************************************
 * $Id::                                                               $
 *
 * Project:	uart: Simple UART echo for LPCXpresso 1700
 * File:	uart.c
 * Description:
 * 			LPCXpresso Baseboard uses pins mapped to UART3 for
 * 			its USB-to-UART bridge. This application simply echos
 * 			all characters received.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/*****************************************************************************
 *   History
 *	 2015.10.10	 ver 1.05	 Added UARTGetLpcUart. UARTGetLpcUart
 *   2010.07.01  ver 1.01    Added support for UART3, tested on LPCXpresso 1700
 *   2009.05.27  ver 1.00    Prelimnary version, first Release
 *
 ******************************************************************************/
#include "LPC17xx.h"
#include "system_LPC17xx.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "uart.h"
#include "printf.h"
#include <assert.h>
#include <string.h>


#define IER_RBR		0x01
#define IER_THRE	0x02
#define IER_RLS		0x04

#define IIR_PEND	0x01
#define IIR_RLS		0x03
#define IIR_RDA		0x02
#define IIR_CTI		0x06
#define IIR_THRE	0x01

#define LSR_RDR		0x01
#define LSR_OE		0x02
#define LSR_PE		0x04
#define LSR_FE		0x08
#define LSR_BI		0x10
#define LSR_THRE	0x20
#define LSR_TEMT	0x40
#define LSR_RXFE	0x80


#define PCUART0_POWERON (1 << 3)
#define PCUART1_POWERON (1 << 4)
#define PCUART2_POWERON (1 << 24)
#define PCUART3_POWERON (1 << 25)

#define PCLK_UART0 6
#define PCLK_UART1 8
#define PCLK_UART2 16
#define PCLK_UART3 18
#define PCLK_UART0_MASK (3 << 6)
#define PCLK_UART1_MASK (3 << 8)
#define PCLK_UART2_MASK (3 << 16)
#define PCLK_UART3_MASK (3 << 18)


extern xSemaphoreHandle PRINTF_semaphores[UART_portNum];

UART_state UART_states[UART_portNum] = {{ 0 }};


UART_state* UART_getState(UART_port port)
{
	UART_state* result = NULL;
	switch(port)
	{
		case UART_port0:
		case UART_port1:
		case UART_port3:
			result = &UART_states[port];
			break;
		default:
			break;
	}
	return result;
}

void UARTx_IRQHandler(UART_state* s)
{
	uint8_t IIRValue, LSRValue;
	uint8_t Dummy = Dummy;

	IIRValue = s->lpc->IIR;

	IIRValue >>= 1; /* skip pending bit in IIR */
	IIRValue &= 0x07; /* check bit 1~3, interrupt identification */
	if (IIRValue == IIR_RLS) /* Receive Line Status */
	{
		LSRValue = s->lpc->LSR;
		/* Receive Line Status */
		if (LSRValue & (LSR_OE | LSR_PE | LSR_FE | LSR_RXFE | LSR_BI))
		{
			/* There are errors or break interrupt */
			/* Read LSR will clear the interrupt */
			s->status = LSRValue;
			Dummy = s->lpc->RBR; /* Dummy read on RX to clear
			 interrupt, then bail out */

			return;
		}
		if (LSRValue & LSR_RDR) /* Receive Data Ready */
		{
			/* If no error on RLS, normal ready, save into the data buffer. */
			/* Note: read RBR will clear the interrupt */
			s->buffer[s->count] = s->lpc->RBR;
			s->count++;
			if (s->count == UART_bufferSize)
			{
				assert(0);	/* TODO: look out for overflows */
				s->count = 0; /* buffer overflow */
			}
		}
	}
	else if (IIRValue == IIR_RDA) /* Receive Data Available */
	{
		/* Receive Data Available */
		s->buffer[s->count] = s->lpc->RBR;
		s->count++;
		if (s->count == UART_bufferSize)
		{
			assert(0);	/* TODO: look out for overflows */
			s->count = 0; /* buffer overflow */
		}
	}
	else if (IIRValue == IIR_CTI) /* Character timeout indicator */
	{
		/* Character Time-out indicator */
		s->status |= 0x100; /* Bit 9 as the CTI error */
	}
	else if (IIRValue == IIR_THRE) /* THRE, transmit holding register empty */
	{
		/* THRE interrupt */
		LSRValue = s->lpc->LSR; /* Check status in the LSR to see if
		 valid data in U0THR or not */
		if (LSRValue & LSR_THRE)
		{
			s->txEmpty = 1;
		}
		else
		{
			s->txEmpty = 0;
		}
	}
}

void UART0_IRQHandler(void)
{
	UARTx_IRQHandler(UART_getState(0));
}
void UART1_IRQHandler(void)
{
	UARTx_IRQHandler(UART_getState(1));
}
void UART3_IRQHandler(void)
{
	UARTx_IRQHandler(UART_getState(3));
}

int UART_isInitialized(UART_port port)
{
	UART_state* s = UART_getState(port);
	return s->mode != UART_modeNotInitialized;
}

/*****************************************************************************
 ** Function name:		UARTInit
 **
 ** Descriptions:		Initialize UART port, setup pin select,
 **						clock, parity, stop bits, FIFO, etc.
 **
 ** parameters:			portNum(0 or 1) and UART baudrate
 ** Returned value:		true or false, return false only if the
 **						interrupt handler can't be installed to the
 **						VIC table
 **
 *****************************************************************************/
/*****************************************************************************
 ** Function name:		UARTInit
 **
 ** Descriptions:		Initialize UART port, setup pin select,
 **						clock, parity, stop bits, FIFO, etc.
 **
 ** parameters:			portNum and UART baudrate
 ** Returned value:		true or false, return false only if the
 **						interrupt handler can't be installed to the
 **						VIC table
 **
 *****************************************************************************/
Result UART_init_with_interrupts(UART_port port, uint32_t baudrate)
{
	xSemaphoreTake(PRINTF_semaphores[port], portMAX_DELAY);
	uint32_t Fdiv;
	uint32_t pclkdiv, pclk;

	Result result = Result_OK;
	UART_state* s = UART_getState(port);
	if(s)
	{
		if (port == 0)
		{
			LPC_PINCON->PINSEL0 &= ~0x000000F0;
			LPC_PINCON->PINSEL0 |= 0x00000050; /* RxD0 is P0.3 and TxD0 is P0.2 */
			/* By default, the PCLKSELx value is zero, thus, the PCLK for
			 all the peripherals is 1/4 of the SystemFrequency. */
			/* Bit 6~7 is for UART0 */
			pclkdiv = (LPC_SC->PCLKSEL0 >> 6) & 0x03;
		}
		else if (port == 1)
		{
	#if 0 // original code
			LPC_PINCON->PINSEL4 &= ~0x0000000F;
			LPC_PINCON->PINSEL4 |= 0x0000000A; /* Enable RxD1 P2.1, TxD1 P2.0 */
	#else // fixed code
			//Enable TxD1 (pin P0.15)
			LPC_PINCON->PINSEL0 &= ~0xC0000000; //P0.15 is bits 31:30
			LPC_PINCON->PINSEL0 |= 0x40000000;  //TxD1 is 01
			//Enable RxD1 (pin P0.16)
			LPC_PINCON->PINSEL1 &= ~0x00000003; //P0.16 is bits 1:0
			LPC_PINCON->PINSEL1 |= 0x00000001;  //RxD1 is 01
	#endif
			pclkdiv = (LPC_SC->PCLKSEL0 >> 8) & 0x03;
		}
		else if(port == 2)
		{
			assert(0);	/* UART2 is not not supported */
		}
		else if (port ==3)
		{
			LPC_PINCON->PINSEL0 &= ~0x0000000F;
			LPC_PINCON->PINSEL0 |= 0x0000000A; /* RxD3 is P0.1 and TxD3 is P0.0 */
			LPC_SC->PCONP |= 1 << 4 | 1 << 25; //Enable PCUART1
			/* By default, the PCLKSELx value is zero, thus, the PCLK for
			 all the peripherals is 1/4 of the SystemFrequency. */
			/* Bit 6~7 is for UART3 */
			pclkdiv = (LPC_SC->PCLKSEL1 >> 18) & 0x03;
		}

		switch (pclkdiv)
		{
		case 0x00:
		default:
			pclk = SystemCoreClock / 4;
			break;
		case 0x01:
			pclk = SystemCoreClock;
			break;
		case 0x02:
			pclk = SystemCoreClock / 2;
			break;
		case 0x03:
			pclk = SystemCoreClock / 8;
			break;
		}

		s->lpc->LCR = 0x83; /* 8 bits, no Parity, 1 Stop bit */
		Fdiv = (pclk / 16) / baudrate; /*baud rate */
		s->lpc->DLM = Fdiv / 256;
		s->lpc->DLL = Fdiv % 256;
		s->lpc->LCR = 0x03; /* DLAB = 0 */
		s->lpc->FCR = 0x07; /* Enable and reset TX and RX FIFO. */

		if (port == 0) 		NVIC_EnableIRQ(UART0_IRQn);
		else if (port ==1) 	NVIC_EnableIRQ(UART1_IRQn);
		else if (port ==3) 	NVIC_EnableIRQ(UART3_IRQn);

		s->lpc->IER = IER_RBR | IER_THRE | IER_RLS; /* Enable UART interrupt */
	}
	else
	{
		result = Result_Failed_InvalidParams;
	}

	xSemaphoreGive(PRINTF_semaphores[port]);
	return result;
}
/////////////////////////////////////////////////////////////////////////////////////
// Alternative implementation, no interrupts
// we use internal 16-byte FIFO
// TODO: undefine unused code to save memory

void UART0_init_no_interrupts(int baudrate)
{
	int pclk;
	unsigned long int Fdiv;

	// PCLK_UART0 is being set to 1/4 of SystemCoreClock
	pclk = SystemCoreClock / 4;

	// Turn on power to UART0
	LPC_SC->PCONP |= PCUART0_POWERON;

	// Turn on UART0 peripheral clock
	LPC_SC->PCLKSEL0 &= ~(PCLK_UART0_MASK);
	LPC_SC->PCLKSEL0 |= (0 << PCLK_UART0);		// PCLK_periph = CCLK/4

	// Set PINSEL0 so that P0.2 = TXD0, P0.3 = RXD0
	LPC_PINCON->PINSEL0 &= ~0xf0;
	LPC_PINCON->PINSEL0 |= ((1 << 4) | (1 << 6));

	LPC_UART0->LCR = 0x83;		// 8 bits, no Parity, 1 Stop bit, DLAB=1
	Fdiv = (pclk / 16) / baudrate;	// Set baud rate
	LPC_UART0->DLM = Fdiv / 256;
	LPC_UART0->DLL = Fdiv % 256;
	LPC_UART0->LCR = 0x03;		// 8 bits, no Parity, 1 Stop bit DLAB = 0
	LPC_UART0->FCR = 0x07;		// Enable and reset TX and RX FIFO
}

void UART1_init_no_interrupts(int baudrate)
{
	int pclk;
	unsigned long int Fdiv;

	// PCLK_UART0 is being set to 1/4 of SystemCoreClock
	pclk = SystemCoreClock / 4;

	// Turn on power to UART0
	LPC_SC->PCONP |= PCUART1_POWERON;

	// Turn on UART0 peripheral clock
	LPC_SC->PCLKSEL0 &= ~(PCLK_UART1_MASK);
	LPC_SC->PCLKSEL0 |= (0 << PCLK_UART1);		// PCLK_periph = CCLK/4

	// Set PINSEL0 so that P0.2 = TXD0, P0.3 = RXD0
	LPC_PINCON->PINSEL0 &= ~0xc0000000;
	LPC_PINCON->PINSEL0 |= (1 << 30);
	LPC_PINCON->PINSEL1 &= ~0xc;
	LPC_PINCON->PINSEL1 |= (1 << 0);

	LPC_UART1->LCR = 0x83;		// 8 bits, no Parity, 1 Stop bit, DLAB=1
	Fdiv = (pclk / 16) / baudrate;	// Set baud rate
	LPC_UART1->DLM = Fdiv / 256;
	LPC_UART1->DLL = Fdiv % 256;
	LPC_UART1->LCR = 0x03;		// 8 bits, no Parity, 1 Stop bit DLAB = 0
	LPC_UART1->FCR = 0x07;		// Enable and reset TX and RX FIFO
}

void UART2_init_no_interrupts(int baudrate)
{
	int pclk;
	unsigned long int Fdiv;

	// PCLK_UART2 is being set to 1/4 of SystemCoreClock
	pclk = SystemCoreClock / 4;

	// Turn on power to UART2
	LPC_SC->PCONP |= PCUART0_POWERON;

	// Turn on UART2 peripheral clock
	LPC_SC->PCLKSEL1 &= ~(PCLK_UART2_MASK);
	LPC_SC->PCLKSEL1 |= (0 << PCLK_UART2);		// PCLK_periph = CCLK/4

	// Set PINSEL0 so that P0.10 = TXD2, P0.11 = RXD2
	LPC_PINCON->PINSEL0 &= ~0xf00000;
	LPC_PINCON->PINSEL0 |= ((1 << 20) | (1 << 22));

	LPC_UART2->LCR = 0x83;		// 8 bits, no Parity, 1 Stop bit, DLAB=1
	Fdiv = (pclk / 16) / baudrate;	// Set baud rate
	LPC_UART2->DLM = Fdiv / 256;
	LPC_UART2->DLL = Fdiv % 256;
	LPC_UART2->LCR = 0x03;		// 8 bits, no Parity, 1 Stop bit DLAB = 0
	LPC_UART2->FCR = 0x07;		// Enable and reset TX and RX FIFO
}

void UART3_init_no_interrupts(int baudrate)
{
	int pclk;
	unsigned long int Fdiv;

	// PCLK_UART2 is being set to 1/4 of SystemCoreClock
	pclk = SystemCoreClock / 4;

	// Turn on power to UART2
	LPC_SC->PCONP |= PCUART3_POWERON;

	// Turn on UART2 peripheral clock
	LPC_SC->PCLKSEL1 &= ~(PCLK_UART3_MASK);
	LPC_SC->PCLKSEL1 |= (0 << PCLK_UART3);		// PCLK_periph = CCLK/4

	// Set PINSEL0 so that P0.0 = TXD3, P0.1 = RXD3
	LPC_PINCON->PINSEL0 &= ~0xf;
	LPC_PINCON->PINSEL0 |= ((1 << 1) | (1 << 3));

	LPC_UART3->LCR = 0x83;		// 8 bits, no Parity, 1 Stop bit, DLAB=1
	Fdiv = (pclk / 16) / baudrate;	// Set baud rate
	LPC_UART3->DLM = Fdiv / 256;
	LPC_UART3->DLL = Fdiv % 256;
	LPC_UART3->LCR = 0x03;		// 8 bits, no Parity, 1 Stop bit DLAB = 0
	LPC_UART3->FCR = 0x07;		// Enable and reset TX and RX FIFO
}

Result UART_init_without_interrupts(UART_port port, uint32_t baudrate)
{
	typedef void (*InitFunc_t)(int);
	InitFunc_t funcs[UART_portNum] =
	{
		UART0_init_no_interrupts,
		UART1_init_no_interrupts,
		UART2_init_no_interrupts,
		UART3_init_no_interrupts
	};
	(*funcs[port])(baudrate);
	return Result_OK;
}

Result UART_init(UART_port port, uint32_t baudrate, UART_mode mode)
{
	if(UART_isInitialized(port))
	{
		return Result_Failed_AlreadyInitialized; // UART already initialized
	}
	if(!PRINTF_isInitialized(port))
	{
		return Result_Failed_NotInitialized; // PRINTF not initialized
	}

	UART_state* s = UART_getState(port);
	if(s == NULL)
	{
		return Result_Failed_InvalidParams;
	}

	memset(s, 0, sizeof(UART_state));
	LPC_UART_TypeDef* lpcs[] = { (void*)LPC_UART0, (void*)LPC_UART1, (void*)LPC_UART2 , (void*)LPC_UART3 };
	s->lpc = lpcs[port];
	s->txEmpty = 1;

	Result result = Result_Failed_InvalidParams;
	switch(mode)
	{
		case UART_modeWithoutInterrupts:
			result = UART_init_without_interrupts(port, baudrate);
			break;
		case UART_modeWithInterrupts:
			result = UART_init_with_interrupts(port, baudrate);
			break;
		default:
			break;
	}
	if(result == Result_OK )
	{
		s->mode = mode;
	}
	return result;
}

Result UART_deinit(UART_port port)
{
	if(!UART_isInitialized(port))
	{
		return Result_Failed_NotInitialized;
	}
	UART_state* s = UART_getState(port);
	if(s->mode == UART_modeWithInterrupts)
	{
		s->lpc->IER = 0; /* Disable UART interrupt - TODO: experimental; so far looks good */
	}
// TODO: deinit x modeWithoutInterrupts ?
	s->mode = UART_modeNotInitialized;
	return Result_OK;						// TODO: add init/deinit tests for everthing that has deinit
}

#if 0
/** Sends data through a port. This is for internal usage.
 *	for parameter details see UARTSend().
 */
void UARTSend( uint32_t portNum, uint8_t *BufferPtr, uint32_t Length )
{
	if ( portNum == 0 )
	{
		while ( Length != 0 )
		{
			/* THRE status, contain valid data */
			while ( !(UART0TxEmpty & 0x01) );
			LPC_UART0->THR = *BufferPtr;
			UART0TxEmpty = 0; /* not empty in the THR until it shifts out */
			BufferPtr++;
			Length--;
		}
	}
	else if (portNum == 1)
	{
		while ( Length != 0 )
		{
			/* THRE status, contain valid data */
			while ( !(UART1TxEmpty & 0x01) );
			LPC_UART1->THR = *BufferPtr;
			UART1TxEmpty = 0; /* not empty in the THR until it shifts out */
			BufferPtr++;
			Length--;
		}
	}
	else if ( portNum == 3 )
	{
		while ( Length != 0 )
		{
			/* THRE status, contain valid data */
			while ( !(UART3TxEmpty & 0x01) );
			LPC_UART3->THR = *BufferPtr;
			UART3TxEmpty = 0; /* not empty in the THR until it shifts out */
			BufferPtr++;
			Length--;
		}
	}
	return;
}
#endif

void UART_send(UART_port port, char* BufferPtr, uint32_t Length)		// TODO: remove this?
{
#if 0
	UART_state* s = UART_getState(port);
	xSemaphoreTake(PRINTF_semaphores[port], portMAX_DELAY);
//	s->lpc->IER = IER_THRE | IER_RLS;//a				/* Disable RBR */
//	s->lpc->IER |= IER_THRE;//b

	while (Length-- > 0)
	{
		while ((s->lpc->LSR & LSR_THRE) == 0) ;	/* Block until tx empty */
		s->lpc->THR = *BufferPtr++;
		while ((s->lpc->LSR & (1UL << 6)) == 0) ;
	}

// 	s->lpc->IER = IER_THRE | IER_RLS | IER_RBR;//a		/* Re-enable RBR */
//	s->lpc->IER &= ~IER_THRE;//b
	xSemaphoreGive(PRINTF_semaphores[port]);
#else // without interrupts

	UART_state* s = UART_getState(port);
	while (Length-- > 0)
	{
		while ((s->lpc->LSR & LSR_THRE) == 0) ;	/* Block until tx empty */
		s->lpc->THR = *BufferPtr++;
		while ((s->lpc->LSR & (1UL << 6)) == 0) ;
	}

#endif
}


/* TODO: for binary data, UART_getCharacterXXX() won't work */

char UART_getCharacterWithTimeout(UART_port port, uint32_t timeoutMs)
{
	assert(UART_isInitialized(port));
	UART_state* s = UART_getState(port);
	uint32_t timeoutTicks = xTaskGetTickCount() + msToTicks(timeoutMs);

	if(s->mode == UART_modeWithInterrupts)
	{
		xSemaphoreTake(PRINTF_semaphores[port], portMAX_DELAY);
		char result = 0;
		while (result == 0 && xTaskGetTickCount() < timeoutTicks)
		{
			if(s->count > 0 )
			{
				result = s->buffer[0];
				(s->count)--;
				memcpy((void*) s->buffer, (void*)s->buffer+1, (size_t) ((s->count) * sizeof(uint8_t)));
			}
			else
			{
				vTaskDelay(msToTicks(5)); /* release cpu */
			}
		}
		xSemaphoreGive(PRINTF_semaphores[port]);
		return result;
	}
	else
	{
		while( (s->lpc->LSR & LSR_RDR) == 0 )
		{
			if(xTaskGetTickCount() > timeoutTicks)
			{
				return 0; /* no char*/
			}
			else
			{
				vTaskDelay(msToTicks(5)); /* release cpu */
			}
		}
		return s->lpc->RBR;
	}
}


/* Peeks for available character in the UART module
 * If such character is available, it is removed from the UART buffer
 *
 * This is meant for internal usage only
 *
 * @param portNum		port use
 *
 * @return				the character read, or 0. (meaning no character available)
 */
char UART_getCharacter(UART_port port)
{
	assert(UART_isInitialized(port));
	UART_state* s = UART_getState(port);

	if(s->mode == UART_modeWithInterrupts)
	{
		xSemaphoreTake(PRINTF_semaphores[port], portMAX_DELAY);

		char result = 0;
		if(s->count > 0 )
		{
			result = s->buffer[0];
			(s->count)--;
			memcpy((void*) s->buffer, (void*) s->buffer+1, (size_t) ((s->count) * sizeof(uint8_t)));
		}

		xSemaphoreGive(PRINTF_semaphores[port]);
		return result;
	}
	else
	{
		while( (s->lpc->LSR & LSR_RDR) == 0 );  /* nothing received so just block */
		return s->lpc->RBR; /* read Receiver Buffer Register */
	}
}


Result UART_clear(UART_port port, uint32_t timeout)
{
	char c = 1;
	while(c)
	{
		c = UART_getCharacterWithTimeout(port, timeout);
		if(c)
		{
//			LOG_log("----clear----> %c", c);
		}
	}
	return c==0 ? Result_OK : Result_Failed_Timeout;
}
