#include "FreeRTOS.h"
#include "task.h"
#include "wifly.h"
#include "uart.h"
#include "utils.h"
#include "parsing.h"
#include "log.h"
#include "printf.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>


UART_port WIFLY_port = UART_portNone;


Result WIFLY_clearBuffer(uint32_t timeout)
{
	Result result = UART_clear(WIFLY_port, timeout);
	while (result != Result_OK)
	{
		result = UART_clear(WIFLY_port, 1000);
	}
	return result;
}

Result WIFLY_send(const char* s)
{
	Result result = PRINTF_print(WIFLY_port, s);
	LOG_log("WIFLY_send: '%s\n', result=%d", s, result);
	return result;
}

Result WIFLY_receive(char* s, size_t size, uint32_t timeout)
{
	memset(s, 0, size);
	Result result = PRINTF_scanFromDevice(WIFLY_port, s, size, timeout);
	LOG_log("WIFLY_receive: '%s' result=%d", s, result);
	return result;
}

Result WIFLY_genericCommand(const char* cmd, char* buffer, size_t bufferSize, uint32_t cmdTimeout)
{
	assert(cmd != buffer);
	if (WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}

	uint32_t clearTime = 1000;

	WIFLY_clearBuffer(100);
	vTaskDelay(msToTicks(500));
	Result r = WIFLY_send("$$$");
	vTaskDelay(msToTicks(500));

	r = WIFLY_receive(buffer, bufferSize, 300);
	if (r != Result_OK || PARSING_checkSubst(buffer, 0, "CMD") != Result_OK)
	{
		WIFLY_clearBuffer(clearTime);
		return Result_Failed;
	}

	WIFLY_clearBuffer(clearTime);		/* TODO: return codes are forgotten!!!! */
	r = WIFLY_send(cmd);
	r = WIFLY_receive(buffer, bufferSize, cmdTimeout);

	WIFLY_clearBuffer(clearTime);
	WIFLY_send("exit\r");
	r = WIFLY_receive(buffer, bufferSize, 1000);
	if (r != Result_OK || PARSING_checkSubst(buffer, 0, "EXIT") != Result_OK)
	{
		WIFLY_clearBuffer(clearTime);
		return Result_Failed;
	}

	WIFLY_clearBuffer(clearTime);
	return Result_OK;
}

/* TODO: ideally, we should HW reset the WiFly. reboot parameter is for a "reboot command" */
Result WIFLY_init(uint32_t port, uint32_t reboot)
{
	if (WIFLY_port != UART_portNone)
	{
		return Result_Failed_AlreadyInitialized;
	}

	enum { bufferSize = 400 };
	char buffer[bufferSize] = { 0 };

	WIFLY_port = port;

	WIFLY_clearBuffer(1000);
	vTaskDelay(msToTicks(500));
	WIFLY_send("$$$");
	vTaskDelay(msToTicks(500));
	WIFLY_clearBuffer(2000);

	WIFLY_send("\r"); /* paranoid handling of partialy entered commands */
	WIFLY_clearBuffer(2000);

	WIFLY_send("set uart mode 0x03\r"); /* disable echo */   			//TODO: review flags
	WIFLY_clearBuffer(2000);

	if(reboot)
	{
		WIFLY_send("reboot\r");
		Result r = WIFLY_receive(buffer, bufferSize, 20000);
		if (r == Result_OK && PARSING_checkSubst(buffer, -9, "*READY*") == Result_OK)
		{
			LOG_log("WIFLY_init: reboot succeeded");
		}
		else
		{
			LOG_log("WIFLY_init: reboot failed");
			return Result_Failed;
		}

		r = WIFLY_genericCommand("set uart mode 0x03\r", buffer, bufferSize, 2000);
		WIFLY_clearBuffer(1000);
	}
	else
	{
		LOG_log("WIFLY_init: reboot skipped");
	}

	LOG_log("WIFLY_init: succeeded");
	return Result_OK;
}

Result WIFLY_deinit(void)
{
	if(WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}
	WIFLY_port = UART_portNone;
	return Result_OK;
}

Result WIFLY_join(const char* ssid, const char* pwd)
{
	if(WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}

	enum { cmdSize = 60, bufferSize = 300, linesSize = 5 };
	char cmd[cmdSize] = { 0 };
	char buffer[bufferSize] = { 0 };  		/* use buffer for both command input and output */
	char* lines[linesSize] = { 0 };

	Result r;
	snprintf(cmd, cmdSize, "set wlan pass %s\r", pwd);
	r = WIFLY_genericCommand(cmd, buffer, bufferSize, 3000);	/*  TODO: should we make versions of WIFLY_command() and LOG_log that take var args? */
	if (r != Result_OK)
	{
		return Result_Failed_InvalidParams;
	}

	snprintf(cmd, cmdSize, "join %s\r", ssid);
	r = WIFLY_genericCommand(cmd, buffer, bufferSize, 20000);
	if(r != Result_OK)
	{
		return r;
	}

	r = PARSING_splitLines(buffer, lines, linesSize);
	if ((  (strcmp(lines[3], "Associated!") == 0)
		|| (strcmp(lines[4], "Associated!") == 0))) /* response contains 'deAuth' */
	{
		return Result_OK;
	}
	return Result_Failed;
}

Result WIFLY_reboot()
{
	if(WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}
	UART_port port = WIFLY_port;
	WIFLY_port = UART_portNone;		/* this is a hack into WIFLY_init() */
	Result r = WIFLY_init(port, 1);
	return r;
}

/*
 * it is responsibility of the caller to make sure that ip param points to a buffer of at very least 17 characters
 * */
Result WIFLY_getIp(char* ip)
{
	if(WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}
	enum { bufferSize = 30 };
	char buffer[bufferSize] = { 0 };

	Result r = WIFLY_genericCommand("get ip a\r", buffer, bufferSize, 1000);	/* remove trailing 'a' to get the port as well */
	if (r == Result_OK)
	{
		PARSING_splitLines(buffer, NULL, 0); /* we'll ignore the Result_Failed_NotEnoughMemory return code */
		strcpy(ip, buffer);
	}
	return r;
}

/*
 * it is responsibility of the caller to make sure that channel param points to a buffer large enough to accommodate the result
 * */
Result WIFLY_getChannel(char* channel)
{
	if(WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}
	enum { bufferSize = 100, linesSize = 2 };
	char buffer[bufferSize] = { 0 };
	char* lines[linesSize] = { 0 };

	Result r = WIFLY_genericCommand("get wlan\r", buffer, bufferSize, 2000);
	if(r == Result_OK)
	{
		r = PARSING_splitLines(buffer, lines, linesSize);
		if (r == Result_OK && lines[1])
		{
			strcpy(channel, PARSING_removeEnclosingTags(lines[1], "Chan=", ""));
			return Result_OK;
		}
	}
	return Result_Failed;
}

Result WIFLY_getRSSI(char* rssi)
{
	if (WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}
	enum { bufferSize = 20 };
	char buffer[bufferSize] = { 0 };

	Result r = WIFLY_genericCommand("show rssi\r", buffer, bufferSize, 2000);
	if(r == Result_OK)
	{
/* TODO: is that space after the '(' a placeholder for a +/- sign? can RSSI be negative? */
		strcpy(rssi, PARSING_removeEnclosingTags(buffer, "RSSI=( ", ") dBm"));
		return Result_OK;
	}
	return Result_Failed;
}

Result WIFLY_sendHttpGetRequest(const char *server, const char* remainderUrl, char* response, size_t responseSize)
{
	if(WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}
/* TODO: validate connection? */
	enum { bufferSize = 200 };
	char buffer[bufferSize] = { 0 };

	snprintf(buffer, bufferSize, "%s%s", server, remainderUrl);
	LOG_log("Sending HTTP GET request: '%s'", buffer);

	Result r = WIFLY_genericCommand("set ip proto 18\r", buffer, bufferSize, 2000);
	if(r != Result_OK || PARSING_checkSubst(buffer, 0, "AOK") != Result_OK)
	{
		return Result_Failed;
	}

	r = WIFLY_genericCommand("set ip address 0\r", buffer, bufferSize, 2000);
	if(r != Result_OK || PARSING_checkSubst(buffer, 0, "AOK") != Result_OK)
	{
		return Result_Failed;
	}

	r = WIFLY_genericCommand("set ip remote 80\r", buffer, bufferSize, 2000);
	if(r != Result_OK || PARSING_checkSubst(buffer, 0, "AOK") != Result_OK)
	{
		return Result_Failed;
	}

	r = WIFLY_genericCommand("set option format 1\r", buffer, bufferSize, 2000);
	if(r != Result_OK || PARSING_checkSubst(buffer, 0, "AOK") != Result_OK)
	{
		return Result_Failed;
	}

	snprintf(buffer, bufferSize, "set dns name %s\r", server);
	r = WIFLY_genericCommand(buffer, buffer, bufferSize, 2000);
	if(r != Result_OK || PARSING_checkSubst(buffer, 0, "AOK") != Result_OK)
	{
		return Result_Failed;
	}

	snprintf(buffer, bufferSize, "set com remote GET$%s\r", remainderUrl);
	r = WIFLY_genericCommand(buffer, buffer, bufferSize, 2000);
	if(r != Result_OK || PARSING_checkSubst(buffer, 0, "AOK") != Result_OK)		/* TODO: unclear if AOK is only response */
	{
		return Result_Failed;
	}

/* TODO: if connection speed is fast enough, we might run into a buffer overflow in UART module */

	r = WIFLY_genericCommand("open\r", response, responseSize, 20000);		/* TODO: what timeout is appropriate? */
	if(r != Result_OK)
	{
		char* open = "*OPEN*";
		char* close = "*CLOS*";
		if(PARSING_checkSubst(response, 0, open) == Result_OK)
		{
			const char* p = PARSING_removeEnclosingTags(response, open, close);
			memmove(response, p, 1 + strlen(p)); /* can't use strcpy() as there is overlap */
			return Result_OK;
		}
	}
	return Result_Failed; /* TODO: can't handle partial responses (in case response is too small) */
}

Result WIFLY_getTxPower(int* txPower)
{
	if(WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}
	enum { bufferSize = 200, linesSize = 12 };
	char buffer[bufferSize] = { 0 };
	char* lines[linesSize] = { 0 };

	Result r = WIFLY_genericCommand("get wlan\r", buffer, bufferSize, 2000);
	if(r == Result_OK)
	{
		r = PARSING_splitLines(buffer, lines, linesSize);
		if(r == Result_OK && lines[10])
		{
			char* txPowerTag = "TxPower=";
			if(PARSING_checkSubst(lines[10], 0, txPowerTag) == Result_OK)
			{
				if(txPower)
				{
					*txPower = atoi(PARSING_removeEnclosingTags(lines[10], txPowerTag, ""));
				}
				return Result_OK;
			}
		}
	}
	return Result_Failed;
}

Result WIFLY_setTxPower(int txPower)
{
	if(WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}
	if(txPower < 0 || txPower >12)
	{
		return Result_Failed_InvalidParams;
	}

	enum { bufferSize = 100 };
	char buffer[bufferSize] = { 0 };

	snprintf(buffer, bufferSize, "set wlan tx %d\r", txPower);
	Result r = WIFLY_genericCommand(buffer, buffer, bufferSize, 2000);
	if(r == Result_OK && PARSING_checkSubst(buffer, 0, "AOK") == Result_OK)
	{
		return Result_OK;
	}
	return Result_Failed;
}

Result WIFLY_getUptime(int* uptime)
{
	if(WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}

	enum { bufferSize = 100, linesSize = 3 };
	char buffer[bufferSize] = { 0 };
	char* lines[linesSize] = { 0 };

	Result r = WIFLY_genericCommand("show time\r", buffer, bufferSize, 1000);
	if(r == Result_OK)
	{
		r = PARSING_splitLines(buffer, lines, linesSize);
		if(r == Result_OK && lines[1])
		{
			*uptime = atoi(PARSING_removeEnclosingTags(lines[1], "UpTime=", " s"));
			return Result_OK;
		}
	}
	return Result_Failed;
}

Result WIFLY_ping(const char* address, const char* value)  /* value is of type char because we might want to abuse Wifly API */
{
	if(WIFLY_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}
	enum { bufferSize = 200 };
	char buffer[bufferSize] = { 0 };

	snprintf(buffer, bufferSize, "ping %s %s\r", address, value);
	Result r = WIFLY_genericCommand(buffer, buffer, bufferSize, 10000);
	if(r != Result_OK || PARSING_checkSubst(buffer, 3, "bytes from") != Result_OK)
	{
		return Result_OK;
	}
	return Result_Failed;
}
