/**Entry point & top level code for the Alarm Test application
 *
 */

#include "test_framework.h"
#include "test_helpers.h"
#include "test_config.h"
#include <assert.h>
#include <string.h>
#include <stdio.h>


/* declarations all tests in tests/xxx.c */
void unitTest_splitLines(void*);
void unitTest_checkSubst(void*);
void unitTest_removeEnclosingTags(void*);
void unitTest_manyCommands(void*);
void unitTest_initWifly(void*);
void unitTest_selfUart(void*);
void unitTest_crashScan(void*);
void unitTest_initGpioInt(void*);
void unitTest_selfGpioInt(void*);
void unitTest_selfGpio(void*);
void noTest_toggleLed(void*);
void unitTest_ledctrlInit(void*);

void noTest_printfBroadcast(void*);
void noTest_timings(void*);
void noTest_palette(void*);

void manualTest_keypad(void*);
void manualTest_gpio(void*);
void manualTest_fsm(void*);
void manualTest_ledctrl(void*);
void manualTest_cmd(void*);
void manualTest_sensors(void*);


/**Remarks:
 * a) General comments, regarding all tests:
 *  - A TEST module has been implemented @ test_framework.
 *    Tests (as defined in "Test test[]") are called as a FreeRTOS task. The void* parameter is never used.
 *    Each test specifies its required stack size, which is validated upon test completion by the framework.
 *  - Some tests require a specific test HW setup. This setup involves a number of things, such as:
 *    Two LEDs (referred to as green and red).
 *    Two sensors (referred to as MC and PIR).
 *    A matrix keypad.
 *    A number of pairs of GPIO pins connected to each other, to validate pin I/O.
 *    Two UARTs interconnecting their Rx/Tx for the analogous purpose.
 *  - Granularity in test vary greatly: some check a single function, some check a complete module or even a subsystem of the alarm application.
 *  - An effort has been made to keep tests to a reasonable number.
 *    Test maintenance is expensive and implementation has undergone many changes.
 *  - Feedback is provided on test checks and progress through the LOG module.
 *    A terminal listening on the logging port is required to observe test progress.
 *    See testConfig_loggerPort and testConfig_loggerBauds for required logger configuration.
 *
 * b) About manual tests:
 *  - Implement smoke/integration/regression testing.
 *  - All manual tests provide user instructions on what to do and what to expect for that specific test.
 *    Instructions are shown in the terminal window, in distint blue color.
 *  - Most tests validate various elements at once.
 *  - Most tests require user intervention:
 *    In particular, most tests make use of the keypad and expect a certain user input to finalize.
 *    Also, some tests require interaction with other elements (such as sensors).
 *    Often, complete validation can't be done by the test frame itself.
 *
 * c) About unit tests or HW tests:
 *  - Test implementation is as per classic unit testing, and some features have been developed using TDD.
 *
 * d) About non-tests:
 *  - Even though they have the form of tests, and they do some validation, this is, more than a test, a mechanism that:
 *    Provides a minimal infrastructure that can be used to validate a given implementation via its interaction/inspection.
 *    Has no exhaustive checks. Thus can be broken without noticing.
 *  - Examples can serve as (current test may not reflect this, but this was useful during development):
 *    Validation that a specific setup compiles.
 *    Code dependency exploration.
 *    HW exploration.
 *    As stress tests.
 *    As framework for TDD.
 */

int main(void)
{
	enum
	{
		prioTest		= tskIDLE_PRIORITY+2,		/* basic test task priority (tip) */
		prioSuite		= tskIDLE_PRIORITY+3,		/* suite task priority */
		stackDepthSuite	= 1*1024					/* suite stack depth */
	};

	static Test tests[] =
	{
#if 1 /*  manual tests */
		{ manualTest_keypad,		"keypad",			1000, NULL, prioTest },
		{ manualTest_gpio,			"gpio",				1000, NULL, prioTest },
		{ manualTest_sensors,		"sensors",		    2000, NULL, prioTest },
		{ manualTest_ledctrl,		"ledctrl",			2000, NULL, prioTest },
		{ manualTest_fsm,			"fsm",				2000, NULL, prioTest },
		{ manualTest_cmd,			"cmd",				2000, NULL, prioTest },
#endif
#if 1 /*  non-tests */
		{ noTest_palette,			"palette",  		1400, NULL, prioTest },
		{ noTest_printfBroadcast,	"printfBroadcast",	1400, NULL, prioTest },
		{ noTest_timings,			"timings",			1400, NULL, prioTest },
#endif
#if 1 /*  unit tests (also, HW tests) */
		{ unitTest_splitLines,		"splitLines",		1400, NULL, prioTest },
		{ unitTest_checkSubst,		"checkSubst",		1400, NULL, prioTest },
		{ unitTest_removeEnclosingTags,	"removeTags",	1400, NULL, prioTest },
		{ unitTest_ledctrlInit,		"ledctrlInit",		1000, NULL, prioTest },
		{ unitTest_selfGpio,		"selfGpio",			1400, NULL, prioTest },
		{ unitTest_selfGpioInt,		"selfGpioInt",		1400, NULL, prioTest },
		{ unitTest_initGpioInt,		"initGpioInt",		1400, NULL, prioTest },
#endif
#if 0 /*  failing tests */
		{ unitTest_selfUart,		"selfUart", 		6000, NULL, prioTest },
		{ unitTest_initWifly,		"initWifly",	     800, NULL, prioTest },
		{ unitTest_manyCommands,	"manyCommands",	    6000, NULL, prioTest },
		{ unitTest_crashScan,		"crashScan",	    6000, NULL, prioTest },
#endif
#if 1 /*  example tests -- kept here for historical reasons */
		{ noTest_toggleLed,			"toggleLed",		1400, NULL, prioTest }
#endif
	};

	static TestSuite testSuite =
	{
		"SUITE_ALARM",
		tests, sizeof(tests)/sizeof(tests[0]),
		stackDepthSuite,			/* stack size for the test framework itself */
		prioSuite,					/* priority of the test executor task */
		testConfig_loggerPort,		/* logger UART port. Setting shared by all tests */
		testConfig_loggerBauds		/* logger baud rate. Setting shared by all tests */
	};

	TEST_runSuite(&testSuite);
	return 0; /* unreachable */
}
