#include "test_framework.h"
#include "test_helpers.h"
#include "FreeRTOS.h"
#include "task.h"
#include "utils.h"
#include "printf.h"
#include "log.h"
#include "uart.h"
#include <assert.h>
#include <string.h>
#include <stdlib.h>


Test* currentTest = NULL;
char TEST_strFail[30]     = "FAILED   "; /* extra room for as additional ansi escap sequences */
char TEST_strSuccess[30]  = "SUCCEEDED";


void TEST_CheckResultFL(Result result, char* file, int line)
{
	LOG_log
	(
		"CHECK-%s(%d) in %s(%d)",
		Result_OK == result ? TEST_strSuccess : TEST_strFail,
		result, file, line
	);

	if(result != Result_OK)
	{
		currentTest->failedChecks++;
	}
	currentTest->totalChecks++;
}

void TEST_CheckIntFL(int iResult, char* file, int line)
{
	Result result = ((iResult != 0) ? Result_OK : Result_Failed);
	TEST_CheckResultFL(result, file, line);
}

void TEST_prepareRun(Test* t)
{
	t->totalChecks = 0;
	t->failedChecks = 0;
	t->handle = 0;
	t->state = TestTaskState_Watiting;
}

void TEST_preRun(Test* t)
{
	char tmp[20];
	currentTest = t;
	LOG_log
	(
		"%sTest '%s' started%s",
		makeAnsiColors(Color_BrightWhite, Color_Black, tmp),
		t->taskName,
		makeAnsiResetAttributes()
	);
}

void TEST_postRun(Test* t)
{
	char tmp[20];

	LOG_log
	(
		"%sTest '%s' completed.%s Result: TEST-%s\r\n",
		makeAnsiColors(Color_BrightWhite, Color_Black, tmp),
		t->taskName,
		makeAnsiResetAttributes(),
		t->failedChecks==0 ? TEST_strSuccess : TEST_strFail
	);
	currentTest = NULL;
}


void TEST_testWrapper(void* params)
{
	Test* test = (Test*) params;

	test->taskProc(test->params);

	uint32_t stackHighWaterMark = wordsToBytes(uxTaskGetStackHighWaterMark(NULL));
	LOG_log("stackHighWaterMark=%d", stackHighWaterMark);
	TEST_CheckInt(stackHighWaterMark > 0);

	currentTest->state = TestTaskState_Completed;
	for(;;)
	{
		vTaskDelay(msToTicks(20));
	}
}

void TEST_run(Test* t)
{
	t->state = TestTaskState_Running;
	uint16_t stackDepth = bytesToWords(t->stackDepth);
	BaseType_t createResult = xTaskCreate(TEST_testWrapper, t->taskName, stackDepth, t, t->priority, &t->handle);

	TEST_CheckInt(createResult == pdTRUE);	/* if this fails, there is not enough memory */
	TEST_CheckInt(t->handle != 0);

	while(t->state != TestTaskState_Completed)
	{
		vTaskDelay(msToTicks(10));
	}
	vTaskDelete(t->handle);
}

void TEST_accumulateResults(TestSuite* testSuite, Test *test)
{
	testSuite->totalChecks	+= test->totalChecks;
	testSuite->failedChecks	+= test->failedChecks;

	testSuite->totalTests++;
	if(test->failedChecks != 0)
	{
		testSuite->failedTests++;
	}
}

void TEST_suiteEnter(TestSuite* testSuite)
{
	makeAnsiString(TEST_strFail,    Color_BrightRed, Color_Black, "FAILED   ");
	makeAnsiString(TEST_strSuccess, Color_Green,     Color_Black, "SUCCEEDED");

	testSuite->totalChecks	= 0;
	testSuite->failedChecks	= 0;
	testSuite->totalTests	= 0;
	testSuite->failedTests	= 0;

	LOG_log("%s%s", makeAnsiClearScreen(), makeAnsiResetAttributes());
	LOG_log("*** SUITE %s started ***", testSuite->name);
}

void TEST_suiteLeave(TestSuite* testSuite)
{
	LOG_log("*** SUITE %s completed ***\n", testSuite->name);
	LOG_log("Summary:");
	LOG_log("    Checks: %d passed out of %d", testSuite->totalChecks - testSuite->failedChecks, testSuite->totalChecks);
	LOG_log("    Tests: %d passed out of %d", testSuite->totalTests - testSuite->failedTests, testSuite->totalTests);
	LOG_log("    Result: SUITE-%s\n", ((testSuite->failedChecks==0 && testSuite->failedTests==0) ? TEST_strSuccess : TEST_strFail));

#if 0		/* process is locked in an infinite loop */
	LOG_log("Done. Hanging device.\n\n");
	for (;;) ;
#else		/* process aborts */
	LOG_log("Done. Aborting.\n\n");
	abort();
#endif
}

void TEST_taskSuite(void* pvParameters)
{
	TestSuite* ts =(TestSuite*) pvParameters;
	Test* t  = ts->tests;

	vTaskDelay(msToTicks(1000));
	Result result = Result_Failed;
	if(ts->logPort != UART_portNone)
	{
		result = PRINTF_init(ts->logPort);
		assert(result == Result_OK);
		result = UART_init(ts->logPort, ts->logPortBauds, UART_modeWithInterrupts);
		assert(result == Result_OK);
		result = LOG_init(ts->logPort);
		assert(result == Result_OK);
	}

	TEST_suiteEnter(ts);

	size_t i;
	for(i=0 ; i < ts->testsSize ; i++)
	{
		TEST_prepareRun(&t[i]);
	}
	for(i=0 ; i < ts->testsSize ; i++)
	{
		TEST_preRun(&t[i]);
		TEST_run(&t[i]);
		TEST_postRun(&t[i]);
		TEST_accumulateResults(ts, &t[i]);
	}

	TEST_suiteLeave(ts);
}

void TEST_runSuite(TestSuite* testSuite)
{
	uint16_t stackDepth = bytesToWords(testSuite->stackDepth);
	BaseType_t createResult = xTaskCreate(TEST_taskSuite, testSuite->name, stackDepth, testSuite, testSuite->priority, NULL);
	assert(createResult == pdTRUE);		/* check for not enough memory */
	vTaskStartScheduler();
	for (;;) ;
}
