#include <assert.h>
#include <stdio.h>
#include "fsm.h"
#include "keypad.h"
#include "utils.h"
#include "log.h"
#include "test_framework.h"
#include "test_config.h"
#include "test_helpers.h"
#include "FreeRTOS.h"
#include "task.h"


/* Support functions for manualTest_fsm() */

static char** GetAllEventsStrings()
{
	static char* r [] =
	{
		"(0) Event_0_unused",
		"(1) Event_Sensor_MagneticContact_On",
		"(2) Event_Sensor_MagneticContact_Off",
		"(3) Event_DisarmUnderCoaction",
		"(4) Event_Sensor_PassiveInfrared_On",
		"(5) Event_Sensor_PassiveInfrared_Off",
		"(6) Event_Disarm",
		"(7) Event_FullyArm",
		"(8) Event_PartialArm",
		"(9) Event_9_unused",
		"(*) Event_Quit (for this test)",
		"(#) Event_None",
		NULL
	};
	return r;
}

static char* getEventString(FSM_Event event)
{
	return GetAllEventsStrings()[event];
};

static FSM_Event getEvent(const KEYPAD_cfg* keypadCfg)
{
	FSM_Event event = Event_None;

	char keys[3*4] = { 0 };
	 KEYPAD_scan(keypadCfg, keys); /* ignore return code */
	char key = KEYPAD_getKeypress(keypadCfg, keys);
	do {	/*wait until key release - this blocks the entire task! */
		KEYPAD_scan(keypadCfg, keys);
	} while(0 != KEYPAD_getKeypress(keypadCfg, keys));

	switch(key)
	{
		case '*':  event = Event_Quit;  break;
		case '#':  event = Event_None;  break;
		default:   event = (FSM_Event)(key - '0');   break;
	}

	if(event != Event_None && event >= 0 && event < Event_MaxValue)
	{
		char tmp[30];
		LOG_log
		(
			"%s%s%s",
			makeAnsiColors(Color_Yellow, Color_Black, tmp),
			getEventString(event),
			makeAnsiResetAttributes()
		);
	}
	else
	{
		event = Event_None;
	}
	return event;
}

static void procesEvent(FSM_Alarm* as, FSM_Event event)
{
	FSM_SensorState* mc  = &as->sensorStateMagneticContact;
	FSM_SensorState* pir = &as->sensorStatePassiveInfrared;
	uint32_t logSensorStates = 1;

	switch(event)
	{
		case Event_Sensor_MagneticContact_On:
			FSM_setDetection(mc, 1);
			break;
		case Event_Sensor_MagneticContact_Off:
			FSM_setDetection(mc, 0);
			break;
		case Event_Sensor_PassiveInfrared_On:
			FSM_setDetection(pir, 1);
			break;
		case Event_Sensor_PassiveInfrared_Off:
			FSM_setDetection(pir, 0);
			break;
		case Event_Quit:
			as->fsmState = State_Quit;
			break;
		default:
			logSensorStates = 0;
			break;
	}

	if(logSensorStates)
	{
		FSM_logSensorStates(as);
	}
}

/* Manual test for the FSM/FSMSTATE modules. See in-test instructions for details */
void manualTest_fsm(void* pvParameters)
{
	assert(pvParameters == NULL);
	char* instructions [] =
	{
		"Manual test for FSM/FSM module.",
		"To finish this test, press '*' in the keypad.",
		"This test keeps track of the alarm state, allowing user to send events.",
		"LEDCTRL, SENSORS, SISCOM, CMD modules/system do not intervene in this test.",
		"",
		"Available events, mapped as '0'-'9' in the keypad, are:",
		NULL
	};
	showInstructions(instructions);
	showInstructions(GetAllEventsStrings());

	FSM_Alarm fsmAlarm = { 0 };

	KEYPAD_init(&testKeypadCfg);

	FSM_init(&fsmAlarm, procesEvent);
	while(fsmAlarm.fsmState != State_Quit)
	{	
		FSM_Event event = getEvent(&testKeypadCfg);
		FSM_update(&fsmAlarm, event);
		vTaskDelay(msToTicks(100));
	}
}
