#include "test_framework.h"
#include "test_helpers.h"
#include "test_config.h"
#include "log.h"
#include "gpioint.h"
#include "utils.h"
#include <assert.h>
#include <string.h>


/* Support functions for unitTest_selfGpioInt() */

static void intHandler(void* params, uint8_t pinValue)
{
	if (params)
	{
		uint32_t* intCount = (uint32_t*) params;
		(*intCount)++;
	}
}

static void MakeFallingEdge(GPIO_Port port, uint32_t pin)
{
	GPIO_setPinValue(port, pin, 1);
	TEST_CheckInt(1 == GPIO_getPinValue(port, pin));
	GPIO_setPinValue(port, pin, 0);
	TEST_CheckInt(0 == GPIO_getPinValue(port, pin));
	LOG_log("falling edge in P%d.%d", port, pin);
}

/**GPIOINT self-test.
 * Having pins interconnected, Falling edges are generated.
 * Validates that external interruptions are generated accordingly.
 */
void unitTest_selfGpioInt(void* pvParameters)
{
	assert(pvParameters == NULL);
	uint32_t ia = testConfig_PinIA;
	uint32_t oa = testConfig_PinOA;
	uint32_t ib = testConfig_PinIB;
	uint32_t ob = testConfig_PinOB;
	GPIO_Port port = testConfig_PortB;

	uint32_t intACount = 0, expectedACount = 2;
	uint32_t intBCount = 0, expectedBCount = 4;

	TEST_CheckResult(GPIO_configPinAsInput(port, ia, PinMode_PullUp));
	TEST_CheckResult(GPIO_configPinAsInput(port, ib, PinMode_PullUp));
	TEST_CheckResult(GPIO_configPinAsOutput(port, oa));
	TEST_CheckResult(GPIO_configPinAsOutput(port, ob));

	GPIO_setPinValue(port, oa, 0);
	GPIO_setPinValue(port, ob, 0);
	TEST_CheckResult(GPIOINT_init());

	GPIOINT_cfg intACfg = { intHandler, &intACount, port, ia, GPIOINT_EdgeKind_Falling };
	GPIOINT_cfg intBCfg = { intHandler, &intBCount, port, ib, GPIOINT_EdgeKind_Falling };
	TEST_CheckResult(GPIOINT_set(GPIOINT_Slot0, &intACfg));
	TEST_CheckResult(GPIOINT_set(GPIOINT_Slot1, &intBCfg));

	uint32_t oa_ = oa;
	uint32_t ob_ = ob;
	uint32_t swapCount;
	for(swapCount = 0; swapCount < 2; swapCount++)
	{
		uint32_t duration = msToTicks(500);
		uint32_t startTicks = xTaskGetTickCount();
		uint32_t edgeBCount = 0;
		uint32_t lastEdgeBCount = 0;

		MakeFallingEdge(port, oa_);

		while((xTaskGetTickCount() - startTicks) < (duration * 4 * 1.2))
		{
			edgeBCount = (xTaskGetTickCount() - startTicks) / duration;
			if(edgeBCount > lastEdgeBCount)
			{
				lastEdgeBCount = edgeBCount;
				MakeFallingEdge(port, ob_);
			}
			vTaskDelay(duration / 10);
		}

		MakeFallingEdge(port, oa_);

		LOG_log("a-count=%d/%d, b-count=%d/%d", intACount, expectedACount, intBCount, expectedBCount);
		TEST_CheckInt(intACount == expectedACount);
		TEST_CheckInt(intBCount == expectedBCount);

		/* setup for the 2nd iteration */
		swap(&oa_, &ob_);
		swap(&expectedACount, &expectedBCount);
		intACount = intBCount = 0;
	}

	TEST_CheckResult(GPIOINT_unset(GPIOINT_Slot0));
	(GPIOINT_unset(GPIOINT_Slot1));
	TEST_CheckResult(GPIOINT_deinit());
}

/* simple GPIOINT API abuse test */
void unitTest_initGpioInt(void* pvParameters)
{
	assert(pvParameters == NULL);
	uint32_t ia = testConfig_PinIA;
	uint32_t ib = testConfig_PinIB;
	GPIO_Port port = testConfig_PortB;

	TEST_CheckInt(GPIOINT_init() == Result_OK);
	TEST_CheckInt(GPIOINT_init() == Result_Failed_AlreadyInitialized);
	TEST_CheckInt(GPIOINT_deinit() == Result_OK);
	TEST_CheckInt(GPIOINT_deinit() == Result_Failed_NotInitialized);

	GPIOINT_Slot slot1 = GPIOINT_Slot3;
	GPIOINT_Slot slot2 = GPIOINT_Slot4;
	GPIOINT_Slot slot3 = GPIOINT_Slot5;
	GPIOINT_cfg cfg1 = { intHandler, NULL, port, ia, GPIOINT_EdgeKind_Falling };
	GPIOINT_cfg cfg2 = { intHandler, NULL, port, ib, GPIOINT_EdgeKind_Falling };
	GPIOINT_cfg cfg3 = { intHandler, NULL, port, ia, GPIOINT_EdgeKind_Falling };

	TEST_CheckInt(GPIOINT_init() == Result_OK);

	TEST_CheckInt(GPIOINT_set(slot1, &cfg1) == Result_OK);
	TEST_CheckInt(GPIOINT_set(slot2, &cfg2) == Result_OK);
	TEST_CheckInt(GPIOINT_set(slot2, &cfg2) == Result_Failed_AlreadyInitialized);
	TEST_CheckInt(GPIOINT_set(slot3, &cfg3) == Result_OK);
	TEST_CheckInt(GPIOINT_unset(slot3) == Result_OK);
	TEST_CheckInt(GPIOINT_unset(slot2) == Result_OK);
	TEST_CheckInt(GPIOINT_unset(slot2) == Result_Failed_NotInitialized);
	TEST_CheckInt(GPIOINT_set(slot2, &cfg1) == Result_OK);
	TEST_CheckInt(GPIOINT_set(slot2, &cfg1) == Result_Failed_AlreadyInitialized);
	TEST_CheckInt(GPIOINT_unset(slot2) == Result_OK);
	TEST_CheckInt(GPIOINT_unset(slot2) == Result_Failed_NotInitialized);
	TEST_CheckInt(GPIOINT_unset(slot1) == Result_OK);

	TEST_CheckInt(GPIOINT_deinit() == Result_OK);
}
