#include "utils.h"
#include "uart.h"
#include "printf.h"
#include "log.h"
#include "parsing.h"
#include "test_framework.h"
#include "test_config.h"
#include <assert.h>
#include <string.h>
#include <stdlib.h>


/* Support functions for unitTest_selfUart() */

static void randomizeBuffer(char* buffer, uint32_t bufferSize, char minChar, char maxChar)
{
	uint32_t i;
	for(i=0 ; i<bufferSize ; i++)
	{
		buffer[i] = minChar + (char)(rand() % (maxChar - minChar + 1));
	}
}

static uint32_t computeCheckSum(char* buffer, uint32_t bufferSize)
{
	uint32_t checksum = 0;
	uint32_t i;
	for(i=0 ; i<bufferSize ; i++)
	{
		checksum += buffer[i];
	}
	return checksum;
}

static uint32_t receiveAndComputeChecksum(UART_port port, uint32_t bufferSize)
{
	uint32_t checksumReceived = 0;
	uint32_t i;
	for(i=0 ; i<bufferSize ; i++)
	{
		char c = UART_getCharacterWithTimeout(port, 100);
		if(c == 0)
		{
			TEST_CheckInt(c != 0);
			break;
		}
		checksumReceived += c;
	}
	return checksumReceived;
}


/**Basic UART test
 * UART roles (sender/receiver) are exchanged.
 * Also, UARTs are configured with and without interrupts on each role.
 */
void unitTest_selfUart(void* pvParameters)
{
	assert(pvParameters == NULL);
	UART_port portA = UART_port1;
	UART_port portB = UART_port3;
	uint32_t baudrate = 28000;
	UART_mode modeA, modeB;

	assert((int)testConfig_loggerPort != portA);
	assert((int)testConfig_loggerPort != portB);

	for(modeA=UART_modeWithoutInterrupts ; modeA<=UART_modeWithInterrupts ; modeA++)
	for(modeB=UART_modeWithoutInterrupts ; modeB<=UART_modeWithInterrupts ; modeB++)
	{
		LOG_log("init A=P%d(int:%d) -> B=P%d(int:%d)", portA, modeA == UART_modeWithInterrupts, portB, modeB == UART_modeWithInterrupts);
		TEST_CheckResult(PRINTF_init(portA));
		TEST_CheckResult(UART_init(portA, baudrate, modeA));
		TEST_CheckResult(PRINTF_init(portB));
		TEST_CheckResult(UART_init(portB, baudrate, modeB));

		uint32_t checksumSent = 0, checksumReceived = 0;
		enum { maxBufferSize = 100 };
		char buffer[maxBufferSize] = { 0 };

		UART_clear(portA,200);
		UART_clear(portB,200);
		{
			enum { bufferSize = 10 }; 	/* w/ interrupts: limit is UART_bufferSize ; w/out interrupts: limit is 16 bytes (UART FIFO size) */
			LOG_log("test 1) A->B x %d", bufferSize);
			randomizeBuffer(buffer, bufferSize, ' ', 'z');
			checksumSent = computeCheckSum(buffer, bufferSize);
			UART_send(portA, buffer, bufferSize);
			checksumReceived = receiveAndComputeChecksum(portB, bufferSize);
			TEST_CheckInt(checksumReceived == checksumSent);
		}

		UART_clear(portA, 200);
		UART_clear(portB, 200);
		{
			enum { bufferSize = 10 };
			LOG_log("test 2) ( ( A->B , B->A ) x 1 ) x %d", bufferSize);
			randomizeBuffer(buffer, bufferSize, ' ', 'z');
			checksumSent = computeCheckSum(buffer, bufferSize);
			int i;
			for(i=0; i<bufferSize ; i++)
			{
				UART_send(portA, &buffer[i], 1);
				UART_send(portB, &buffer[i], 1);
			}

			checksumReceived = receiveAndComputeChecksum(portA, bufferSize);
			TEST_CheckInt(checksumReceived == checksumSent);
			checksumReceived = receiveAndComputeChecksum(portB, bufferSize);
			TEST_CheckInt(checksumReceived == checksumSent);
		}

		LOG_log("deinit");
		TEST_CheckResult(UART_deinit(portA));
		TEST_CheckResult(PRINTF_deinit(portA));
		TEST_CheckResult(UART_deinit(portB));
		TEST_CheckResult(PRINTF_deinit(portB));
	}
}

